"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_aug.layers import base
from keras_aug.layers import composition
from keras_aug.layers import vision



import keras

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class Identity(VisionRandomLayer):
    """Applies nothing to the inputs."""

    def __init__(self, **kwargs):
        super().__init__(has_generator=False, **kwargs)

    def compute_output_shape(self, input_shape):
        return input_shape

    def augment_images(self, images, transformations, **kwargs):
        return images

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(self, bounding_boxes, transformations, **kwargs):
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        return segmentation_masks

    def augment_keypoints(self, keypoints, transformations, **kwargs):
        return keypoints

    def get_config(self):
        return super().get_config()



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_aug._src.visualization.draw_bounding_boxes import draw_bounding_boxes
from keras_aug._src.visualization.draw_segmentation_masks import (
    draw_segmentation_masks,
)



import keras
import numpy as np
from absl.testing import parameterized
from keras import backend
from keras.src import testing
from keras.src.testing.test_utils import named_product

from keras_aug._src.layers.vision.mosaic import Mosaic
from keras_aug._src.utils.test_utils import get_images


class FixedMosaic(Mosaic):
    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        return dict(
            centers_x=ops.numpy.ones([batch_size]) * 0.75,
            centers_y=ops.numpy.ones([batch_size]) * 0.75,
        )


class MosaicTest(testing.TestCase, parameterized.TestCase):
    def setUp(self):
        # Defaults to channels_last
        self.data_format = backend.image_data_format()
        backend.set_image_data_format("channels_last")
        return super().setUp()

    def tearDown(self) -> None:
        backend.set_image_data_format(self.data_format)
        return super().tearDown()

    @parameterized.named_parameters(named_product(dtype=["float32", "uint8"]))
    def test_correctness(self, dtype):
        # Test channels_last
        images = get_images(dtype, "channels_last")
        inputs = [np.rot90(images, k=i, axes=(1, 2)) for i in range(4)]
        layer = FixedMosaic(size=(32, 32), dtype=dtype)
        outputs = layer(*inputs)

        self.assertDType(outputs, dtype)
        self.assertAllClose(outputs, inputs[0])

        # Test channels_first
        backend.set_image_data_format("channels_first")
        images = get_images(dtype, "channels_first")
        inputs = [np.rot90(images, k=i, axes=(2, 3)) for i in range(4)]
        layer = FixedMosaic(size=(32, 32), dtype=dtype)
        outputs = layer(*inputs)

        self.assertDType(outputs, dtype)
        self.assertAllClose(outputs, inputs[0])

    def test_shape(self):
        # Test dynamic shape
        x = [keras.KerasTensor((None, None, None, 3)) for _ in range(4)]
        y = Mosaic(size=(32, 32))(*x)
        self.assertEqual(y.shape, (None, 32, 32, 3))

        # Test static shape
        x = [keras.KerasTensor((None, 32, 32, 3)) for _ in range(4)]
        y = Mosaic(size=(32, 32))(*x)
        self.assertEqual(y.shape, (None, 32, 32, 3))

    def test_model(self):
        layer = Mosaic(size=(32, 32))
        inputs = [keras.layers.Input(shape=(None, None, 3)) for _ in range(4)]
        outputs = layer(*inputs)
        model = keras.models.Model(inputs, outputs)
        self.assertEqual(model.output_shape, (None, 32, 32, 3))

    def test_config(self):
        x = get_images("float32", "channels_last")
        x = [np.rot90(x, k=i, axes=(1, 2)) for i in range(4)]
        layer = FixedMosaic(size=(32, 32))
        y = layer(*x)

        layer = FixedMosaic.from_config(layer.get_config())
        y2 = layer(*x)
        self.assertAllClose(y, y2)

    def test_tf_data_compatibility(self):
        import tensorflow as tf

        layer = Mosaic(size=(32, 32))
        x = get_images("float32", "channels_last")

        def duplicate(x):
            return x, x, x, x

        ds = (
            tf.data.Dataset.from_tensor_slices(x)
            .map(duplicate)
            .batch(2)
            .map(layer)
        )
        for output in ds.take(1):
            self.assertIsInstance(output, tf.Tensor)
            self.assertEqual(output.shape, (2, 32, 32, 3))

    def test_augment_bounding_box(self):
        from keras_aug._src.layers.vision.random_rotation import RandomRotation

        # Test full bounding boxes
        images = np.zeros([1, 32, 32, 3]).astype("float32")
        boxes = {
            "boxes": np.array(
                [
                    [[0, 0, 1, 1], [0, 0, 1, 1], [0, 0, 1, 1], [0, 0, 1, 1]],
                ],
                "float32",
            ),
            "classes": np.array([[0, 0, 0, 0]], "float32"),
        }
        input = {"images": images, "bounding_boxes": boxes}
        input_r90 = RandomRotation([90, 90], bounding_box_format="rel_xyxy")(
            input
        )
        input_r180 = RandomRotation([180, 180], bounding_box_format="rel_xyxy")(
            input
        )
        input_r270 = RandomRotation([270, 270], bounding_box_format="rel_xyxy")(
            input
        )
        layer = FixedMosaic(size=(32, 32), bounding_box_format="rel_xyxy")

        output = layer(input, input_r90, input_r180, input_r270)
        self.assertAllClose(
            output["bounding_boxes"]["boxes"][0, :4], boxes["boxes"][0]
        )
        self.assertAllClose(
            output["bounding_boxes"]["classes"][0, :4], boxes["classes"][0]
        )

        # Test arbitrary bounding boxes
        images = np.zeros([1, 32, 32, 3]).astype("float32")
        boxes = {
            "boxes": np.array(
                [
                    [
                        [3, 4, 17, 18],
                        [10, 12, 16, 19],
                        [0, 0, 1, 1],
                        [15, 12, 17, 20],
                    ],
                ],
                "float32",
            ),
            "classes": np.array([[0, 1, 2, 3]], "float32"),
        }
        input = {"images": images, "bounding_boxes": boxes}
        layer = FixedMosaic(size=(32, 32), bounding_box_format="xyxy")

        output = layer(input, input, input, input)
        self.assertAllClose(
            output["bounding_boxes"]["boxes"][0, :4], boxes["boxes"][0]
        )
        self.assertAllClose(
            output["bounding_boxes"]["classes"][0, :4], boxes["classes"][0]
        )

    def test_augment_segmentation_mask(self):
        num_classes = 8
        images_shape = (1, 32, 32, 3)
        masks_shape = (1, 32, 32, 1)
        images = np.random.uniform(size=images_shape).astype("float32")
        masks = np.random.randint(2, size=masks_shape) * (num_classes - 1)
        inputs = {"images": images, "segmentation_masks": masks}

        # Crop to exactly 1/2 of the size
        layer = FixedMosaic(size=(32, 32))
        output = layer(inputs, inputs, inputs, inputs)
        self.assertAllClose(output["segmentation_masks"], masks)



import typing

import keras
from keras import backend

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_data_format


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class CutMix(VisionRandomLayer):
    """Apply CutMix to the provided batch of images and labels.

    Note that `CutMix` is meant to be used on batches of inputs, not individual
    input. The sample pairing is deterministic and done by matching consecutive
    samples in the batch, so the batch needs to be shuffled.

    Typically, `CutMix` expects the `labels` to be one-hot-encoded format. If
    they are not, with provided `num_classes`, this layer will transform the
    `labels` into one-hot-encoded format. (e.g. `(batch_size, num_classes)`)

    References:
    - [CutMix: Regularization Strategy to Train Strong Classifiers with Localizable Features](https://arxiv.org/abs/1905.04899)

    Args:
        alpha: The hyperparameter of the beta distribution used for cutmix.
            Defaults to `1.0`.
        num_classes: The number of classes in the inputs. Used for one-hot
            encoding. Can be `None` if the labels are already one-hot-encoded.
            Defaults to `None`.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """  # noqa: E501

    def __init__(
        self,
        alpha: float = 1.0,
        num_classes: typing.Optional[int] = None,
        data_format: typing.Optional[str] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.alpha = float(alpha)
        self.num_classes = int(num_classes) if num_classes is not None else None
        self.data_format = standardize_data_format(data_format)

        if self.data_format == "channels_last":
            self.h_axis, self.w_axis = -3, -2
        else:
            self.h_axis, self.w_axis = -2, -1

    def compute_output_shape(self, input_shape):
        return input_shape

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        random_generator = self.random_generator
        images_shape = ops.shape(images)
        height, width = images_shape[self.h_axis], images_shape[self.w_axis]

        dtype = backend.result_type(self.compute_dtype, float)
        lam = ops.random.beta(
            [batch_size], self.alpha, self.alpha, seed=random_generator
        )
        lam = ops.cast(lam, dtype)
        r_x = ops.random.randint([batch_size], 0, width, seed=random_generator)
        r_x = ops.cast(r_x, dtype)
        r_y = ops.random.randint([batch_size], 0, height, seed=random_generator)
        r_y = ops.cast(r_y, dtype)

        r = 0.5 * ops.numpy.sqrt(1.0 - lam)
        r_w_half = ops.numpy.floor(ops.numpy.multiply(r, width))
        r_h_half = ops.numpy.floor(ops.numpy.multiply(r, height))

        top = ops.numpy.maximum(ops.numpy.subtract(r_y, r_h_half), 0)
        left = ops.numpy.maximum(ops.numpy.subtract(r_x, r_w_half), 0)
        bottom = ops.numpy.minimum(ops.numpy.add(r_y, r_h_half), height)
        right = ops.numpy.minimum(ops.numpy.add(r_x, r_w_half), width)

        area = ops.cast(ops.numpy.multiply(width, height), dtype)
        lam_adjusted = ops.numpy.subtract(
            1.0, (right - left) * (bottom - top) / area
        )
        return dict(
            top=top, bottom=bottom, left=left, right=right, lam=lam_adjusted
        )

    def augment_images(self, images, transformations, **kwargs):
        ops = self.backend

        left = transformations["left"]
        top = transformations["top"]
        right = transformations["right"]
        bottom = transformations["bottom"]
        images = self.image_backend.fill_rectangles(
            images,
            ops.numpy.roll(images, shift=1, axis=0),
            (left, top, right, bottom),
            data_format=self.data_format,
        )
        return images

    def augment_labels(self, labels, transformations, **kwargs):
        ops = self.backend

        lam = transformations["lam"]
        compute_dtype = backend.result_type(labels.dtype, float)
        labels_ndim = len(ops.shape(labels))
        if labels_ndim == 1:
            if self.num_classes is None:
                raise ValueError(
                    "If `labels` is not one-hot-encoded, you must provide "
                    "`num_classes` in the constructor. "
                    f"Received: num_classes={self.num_classes}"
                )
            labels = ops.nn.one_hot(
                labels, self.num_classes, axis=-1, dtype=compute_dtype
            )
        labels = ops.cast(labels, compute_dtype)
        rolled_labels = ops.numpy.roll(labels, shift=1, axis=0)
        lam = ops.numpy.expand_dims(lam, axis=-1)
        labels = ops.numpy.add(
            ops.numpy.multiply(rolled_labels, 1.0 - lam),
            ops.numpy.multiply(labels, lam),
        )
        return labels

    def get_config(self):
        config = super().get_config()
        config.update({"alpha": self.alpha, "num_classes": self.num_classes})
        return config



import typing

import keras
from keras import backend
from keras import saving
from keras.src.utils.backend_utils import in_tf_graph

from keras_aug._src.backend.dynamic_backend import DynamicBackend
from keras_aug._src.backend.dynamic_backend import DynamicRandomGenerator
from keras_aug._src.keras_aug_export import keras_aug_export


@keras_aug_export(parent_path=["keras_aug.layers.composition"])
@keras.saving.register_keras_serializable(package="keras_aug")
class RandomChoice(keras.Layer):
    """Apply single transformation randomly picked from a list.

    Note that due to implementation limitations, the randomness occurs in a
    batch manner.

    Args:
        transforms: A list of transformations or a `keras.Layer`.
        p: A list of probability of each transform being picked. If p doesn't
            sum to `1.0`, it is automatically normalized. If `None`, all
            transforms have the same probability. Defaults to `None`.
    """

    def __init__(
        self,
        transforms,
        p: typing.Optional[typing.Sequence[keras.Layer]] = None,
        seed=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self._backend = DynamicBackend(backend.backend())
        self._random_generator = DynamicRandomGenerator(
            backend.backend(), seed=seed
        )
        self.seed = seed

        # Check
        if not isinstance(transforms, (typing.Sequence, keras.Layer)):
            raise ValueError(
                "`transforms` must be a sequence (e.g. tuple and list) or a "
                "`keras.Layer`. "
                f"Received: transforms={transforms} of type {type(transforms)}"
            )
        if isinstance(transforms, keras.Layer):
            transforms = [transforms]
        if p is not None:
            if not isinstance(p, typing.Sequence):
                raise TypeError(
                    "If `p` is provided, it must be a sequence. "
                    f"Received: p={p} of type {type(p)}"
                )
            if len(p) != len(transforms):
                raise ValueError(
                    "If `p` is provided, the length of it should be the same "
                    "`transforms`. "
                    f"Received: transforms={transforms}, p={p}"
                )
        else:
            p = [1.0] * len(transforms)

        self.transforms = list(transforms)
        total = sum(p)
        self.p = [prob / total for prob in p]

        self._convert_input_args = False
        self._allow_non_tensor_positional_args = True
        self.autocast = False

    @property
    def backend(self):
        return self._backend.backend

    @property
    def random_generator(self):
        return self._random_generator.random_generator

    def compute_output_shape(self, input_shape):
        transform_shape = [
            transfrom.compute_output_shape(input_shape)
            for transfrom in self.transforms
        ]
        transform_shape = set(transform_shape)
        if len(transform_shape) > 1:
            raise ValueError(
                "The output shape of all `transforms` must be the same. "
                f"Received: input_shape={input_shape}, "
                f"possible transform_shape={list(transform_shape)}"
            )
        output_shape = list(transform_shape)[0]
        return output_shape

    def get_params(self):
        ops = self.backend
        random_generator = self.random_generator

        p = ops.convert_to_tensor([self.p])
        p = ops.random.categorical(ops.numpy.log(p), 1, seed=random_generator)
        p = p[0][0]
        return p

    def __call__(self, inputs, **kwargs):
        if in_tf_graph():
            self._set_backend("tensorflow")
            try:
                outputs = super().__call__(inputs, **kwargs)
            finally:
                self._reset_backend()
            return outputs
        else:
            return super().__call__(inputs, **kwargs)

    def call(self, inputs):
        ops = self.backend
        p = self.get_params()

        outputs = ops.core.switch(p, self.transforms, inputs)
        return outputs

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "transforms": saving.serialize_keras_object(self.transforms),
                "p": self.p,
                "seed": self.seed,
            }
        )
        return config

    @classmethod
    def from_config(cls, config, custom_objects=None):
        config = config.copy()
        config["transforms"] = saving.deserialize_keras_object(
            config["transforms"], custom_objects=custom_objects
        )
        return cls(**config)

    def _set_backend(self, name):
        self._backend.set_backend(name)
        self._random_generator.set_generator(name)

    def _reset_backend(self):
        self._backend.reset()
        self._random_generator.reset()



import typing

import keras
from keras import backend

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_parameter


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class RandomHSV(VisionRandomLayer):
    """Randomly change the hue, saturation and value.

    The input images must be 3 channels.

    References:
    - [ultralytics/ultralytics](https://github.com/ultralytics/ultralytics)

    Args:
        hue: How much to jitter hue. The gain will be chosen
            uniformly from `[1 - hue, 1 + hue]` or
            `[min, max]` if given the range of hue. Set to `None` to
            deactivate hue jittering. Defaults to `0.015`.
        saturation: How much to jitter saturation. The gain will be chosen
            uniformly from `[1 - saturation, 1 + saturation]` or
            `[min, max]` if given the range of saturation. Set to `None` to
            deactivate saturation jittering. Defaults to `0.7`.
        value: How much to jitter value. The gain will be chosen
            uniformly from `[1 - value, 1 + value]` or
            `[min, max]` if given the range of value. Set to `None` to
            deactivate value jittering. Defaults to `0.4`.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(
        self,
        hue: typing.Union[None, float, typing.Sequence[float]] = 0.015,
        saturation: typing.Union[None, float, typing.Sequence[float]] = 0.7,
        value: typing.Union[None, float, typing.Sequence[float]] = 0.4,
        data_format=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.hue = standardize_parameter(
            hue, "hue", center=1.0, bound=(0, float("inf"))
        )
        self.saturation = standardize_parameter(
            saturation, "saturation", center=1.0, bound=(0, float("inf"))
        )
        self.value = standardize_parameter(
            value, "value", center=1.0, bound=(0, float("inf"))
        )
        self.data_format = data_format or keras.config.image_data_format()

        if self.hue == (1.0, 1.0):
            self.hue = None
        if self.saturation == (1.0, 1.0):
            self.saturation = None
        if self.value == (1.0, 1.0):
            self.value = None

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        random_generator = self.random_generator

        h, s, v = None, None, None

        def generate_params(low, high):
            return ops.random.uniform(
                [batch_size], low, high, seed=random_generator
            )

        if self.hue is not None:
            h = generate_params(self.hue[0], self.hue[1])
        if self.saturation is not None:
            s = generate_params(self.saturation[0], self.saturation[1])
        if self.value is not None:
            v = generate_params(self.value[0], self.value[1])

        return dict(
            hue_gain=h,
            saturation_gain=s,
            value_gain=v,
        )

    def compute_output_shape(self, input_shape):
        return input_shape

    def augment_images(self, images, transformations=None, **kwargs):
        ops = self.backend

        hue_gain = transformations["hue_gain"]
        saturation_gain = transformations["saturation_gain"]
        value_gain = transformations["value_gain"]
        c_axis = -1 if self.data_format == "channels_last" else -3
        original_dtype = backend.standardize_dtype(images.dtype)
        images = self.image_backend.transform_dtype(
            images, images.dtype, backend.result_type(original_dtype, float)
        )
        images = ops.image.rgb_to_hsv(images, data_format=self.data_format)
        hue, saturation, value = ops.numpy.split(images, 3, axis=c_axis)
        if hue_gain is not None:
            hue_gain = ops.numpy.expand_dims(hue_gain, axis=(1, 2, 3))
            hue = ops.numpy.mod(ops.numpy.multiply(hue, hue_gain), 1.0)
        if saturation_gain is not None:
            saturation_gain = ops.numpy.expand_dims(
                saturation_gain, axis=(1, 2, 3)
            )
            saturation = ops.numpy.multiply(saturation, saturation_gain)
        if value_gain is not None:
            value_gain = ops.numpy.expand_dims(value_gain, axis=(1, 2, 3))
            value = ops.numpy.multiply(value, value_gain)
        images = ops.numpy.concatenate([hue, saturation, value], axis=c_axis)
        images = ops.numpy.clip(images, 0.0, 1.0)
        images = ops.image.hsv_to_rgb(images, data_format=self.data_format)
        images = self.image_backend.transform_dtype(
            images, images.dtype, original_dtype
        )
        return images

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(self, bounding_boxes, transformations, **kwargs):
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        return segmentation_masks

    def augment_keypoints(self, keypoints, transformations, **kwargs):
        return keypoints

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "hue": self.hue,
                "saturation": self.saturation,
                "value": self.value,
            }
        )
        return config



import keras
from absl.testing import parameterized
from keras import backend
from keras import ops
from keras.src import testing
from keras.src.testing.test_utils import named_product

from keras_aug._src.layers.vision.random_hsv import RandomHSV
from keras_aug._src.utils.test_utils import get_images


class FixedRandomHSV(RandomHSV):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Set to non-None
        self.hue = (0.9, 1.1)
        self.saturation = (0.9, 1.1)
        self.value = (0.9, 1.1)

    def get_params(self, batch_size, images=None, **kwargs):
        return dict(
            hue_gain=ops.ones([batch_size]) * 0.9,
            saturation_gain=ops.ones([batch_size]) * 1.1,
            value_gain=ops.ones([batch_size]) * 0.9,
        )


class RandomHSVTest(testing.TestCase, parameterized.TestCase):
    regular_args = dict(hue=0.015, saturation=0.7, value=0.4)

    def setUp(self):
        # Defaults to channels_last
        self.data_format = backend.image_data_format()
        backend.set_image_data_format("channels_last")
        return super().setUp()

    def tearDown(self) -> None:
        backend.set_image_data_format(self.data_format)
        return super().tearDown()

    @parameterized.named_parameters(named_product(dtype=["float32", "uint8"]))
    def test_correctness(self, dtype):
        # Test channels_last
        x = get_images(dtype, "channels_last")
        layer = FixedRandomHSV(dtype=dtype)
        y = layer(x)

        # TODO: Test correctness
        self.assertEqual(y.shape, x.shape)
        self.assertDType(y, dtype)

        # Test channels_first
        backend.set_image_data_format("channels_first")
        x = get_images(dtype, "channels_first")
        layer = FixedRandomHSV(dtype=dtype)
        y = layer(x)

        # TODO: Test correctness
        self.assertEqual(y.shape, x.shape)
        self.assertDType(y, dtype)

    def test_shape(self):
        # Test channels_last
        x = keras.KerasTensor((None, None, None, 3))
        y = RandomHSV(**self.regular_args)(x)
        self.assertEqual(y.shape, (None, None, None, 3))

        # Test channels_first
        backend.set_image_data_format("channels_first")
        x = keras.KerasTensor((None, 3, None, None))
        y = RandomHSV(**self.regular_args)(x)
        self.assertEqual(y.shape, (None, 3, None, None))

        # Test static shape
        backend.set_image_data_format("channels_last")
        x = keras.KerasTensor((None, 32, 32, 3))
        y = RandomHSV(**self.regular_args)(x)
        self.assertEqual(y.shape, (None, 32, 32, 3))

    def test_model(self):
        layer = RandomHSV(**self.regular_args)
        inputs = keras.layers.Input(shape=(None, None, 3))
        outputs = layer(inputs)
        model = keras.models.Model(inputs, outputs)
        self.assertEqual(model.output_shape, (None, None, None, 3))

    def test_data_format(self):
        # Test channels_last
        x = get_images("float32", "channels_last")
        layer = RandomHSV(**self.regular_args)
        y = layer(x)
        self.assertEqual(tuple(y.shape), (2, 32, 32, 3))

        # Test channels_first
        backend.set_image_data_format("channels_first")
        x = get_images("float32", "channels_first")
        layer = RandomHSV(**self.regular_args)
        y = layer(x)
        self.assertEqual(tuple(y.shape), (2, 3, 32, 32))

    def test_config(self):
        x = get_images("float32", "channels_last")
        layer = FixedRandomHSV()
        y = layer(x)

        layer = FixedRandomHSV.from_config(layer.get_config())
        y2 = layer(x)
        self.assertAllClose(y, y2)

    def test_tf_data_compatibility(self):
        import tensorflow as tf

        layer = RandomHSV(**self.regular_args)
        x = get_images("float32", "channels_last")
        ds = tf.data.Dataset.from_tensor_slices(x).batch(2).map(layer)
        for output in ds.take(1):
            self.assertIsInstance(output, tf.Tensor)
            self.assertEqual(output.shape, (2, 32, 32, 3))



from keras.src.utils.backend_utils import in_tf_graph

from keras_aug._src.backend.image import ImageBackend
from keras_aug._src.keras_aug_export import keras_aug_export


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def transform_dtype(images, from_dtype, to_dtype):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).transform_dtype(images, from_dtype, to_dtype)


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def crop(images, top, left, height, width, data_format=None):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).crop(
        images, top, left, height, width, data_format=data_format
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def pad(
    images, mode, top, bottom, left, right, constant_value=0, data_format=None
):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).pad(
        images,
        mode,
        top,
        bottom,
        left,
        right,
        constant_value=constant_value,
        data_format=data_format,
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def adjust_brightness(images, factor):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).adjust_brightness(images, factor)


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def adjust_contrast(images, factor, data_format=None):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).adjust_contrast(
        images, factor, data_format=data_format
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def adjust_saturation(images, factor, data_format=None):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).adjust_saturation(
        images, factor, data_format=data_format
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def adjust_hue(images, factor, data_format=None):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).adjust_hue(
        images, factor, data_format=data_format
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def affine(
    images,
    angle,
    translate_x,
    translate_y,
    scale,
    shear_x,
    shear_y,
    center_x=None,
    center_y=None,
    interpolation="bilinear",
    padding_mode="constant",
    padding_value=0,
    data_format=None,
):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).affine(
        images,
        angle,
        translate_x,
        translate_y,
        scale,
        shear_x,
        shear_y,
        center_x=center_x,
        center_y=center_y,
        interpolation=interpolation,
        padding_mode=padding_mode,
        padding_value=padding_value,
        data_format=data_format,
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def auto_contrast(images, data_format=None):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).auto_contrast(images, data_format=data_format)


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def blend(images1, images2, factor):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).blend(images1, images2, factor)


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def equalize(images, bins=256, data_format=None):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).equalize(
        images, bins=bins, data_format=data_format
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def guassian_blur(images, kernel_size, sigma, data_format=None):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).guassian_blur(
        images, kernel_size, sigma, data_format=data_format
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def rgb_to_grayscale(images, num_channels=3, data_format=None):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).rgb_to_grayscale(
        images, num_channels=num_channels, data_format=data_format
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def invert(images):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).invert(images)


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def posterize(images, bits):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).posterize(images, bits)


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def sharpen(images, factor, data_format=None):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).sharpen(
        images, factor, data_format=data_format
    )


@keras_aug_export(parent_path=["keras_aug.ops.image"])
def solarize(images, threshold):
    backend = "tensorflow" if in_tf_graph() else None
    return ImageBackend(backend).solarize(images, threshold)



import keras
import numpy as np
import pytest
from absl.testing import parameterized
from keras import backend
from keras.src import testing
from keras.src.testing.test_utils import named_product

from keras_aug._src.layers.vision.max_bounding_box import MaxBoundingBox
from keras_aug._src.utils.test_utils import get_images


class MaxBoundingBoxTest(testing.TestCase, parameterized.TestCase):
    def setUp(self):
        # Defaults to channels_last
        self.data_format = backend.image_data_format()
        backend.set_image_data_format("channels_last")
        return super().setUp()

    def tearDown(self) -> None:
        backend.set_image_data_format(self.data_format)
        return super().tearDown()

    @parameterized.named_parameters(named_product(dtype=["float32", "uint8"]))
    def test_correctness(self, dtype):
        inputs = {
            "images": get_images(dtype, "channels_last"),
            "bounding_boxes": {
                "boxes": np.ones((2, 4, 4)),
                "classes": np.ones((2, 4)),
            },
        }
        layer = MaxBoundingBox(max_number=8, dtype=dtype)
        outputs = layer(inputs)
        self.assertAllClose(
            outputs["bounding_boxes"]["boxes"][:, :4, :],
            inputs["bounding_boxes"]["boxes"],
        )
        self.assertAllClose(
            outputs["bounding_boxes"]["classes"][:, :4],
            inputs["bounding_boxes"]["classes"],
        )
        self.assertAllClose(
            outputs["bounding_boxes"]["boxes"][:, 4:, :],
            np.ones((2, 4, 4)) * -1,
        )
        self.assertAllClose(
            outputs["bounding_boxes"]["classes"][:, 4:],
            np.ones((2, 4)) * -1,
        )

    def test_shape(self):
        # Test static shape
        x = {
            "images": keras.KerasTensor((None, 32, 32, 3)),
            "bounding_boxes": {
                "boxes": keras.KerasTensor((None, 4, 4)),
                "classes": keras.KerasTensor((None, 4)),
            },
        }
        y = MaxBoundingBox(max_number=8)(x)
        self.assertEqual(y["bounding_boxes"]["boxes"].shape, (None, 8, 4))
        self.assertEqual(y["bounding_boxes"]["classes"].shape, (None, 8))

    @pytest.mark.skip("keras.models.Model doesn't support nested inputs")
    def test_model(self):
        layer = MaxBoundingBox(max_number=8)
        inputs = {
            "images": keras.KerasTensor((None, 32, 32, 3)),
            "bounding_boxes": {
                "boxes": keras.KerasTensor((None, 4, 4)),
                "classes": keras.KerasTensor((None, 4)),
            },
        }
        outputs = layer(inputs)
        model = keras.models.Model(inputs, outputs)
        self.assertEqual(
            model.output_shape["bounding_boxes"]["boxes"].shape, (None, 8, 4)
        )
        self.assertEqual(
            model.output_shape["bounding_boxes"]["classes"].shape, (None, 8)
        )

    def test_config(self):
        x = get_images("float32", "channels_last")
        inputs = {
            "images": x,
            "bounding_boxes": {
                "boxes": np.ones((2, 4, 4)),
                "classes": np.ones((2, 4)),
            },
        }
        layer = MaxBoundingBox(max_number=8)
        outputs = layer(inputs)
        boxes = keras.ops.convert_to_numpy(outputs["bounding_boxes"]["boxes"])
        classes = keras.ops.convert_to_numpy(
            outputs["bounding_boxes"]["classes"]
        )

        layer = MaxBoundingBox.from_config(layer.get_config())
        outputs2 = layer(inputs)
        boxes2 = keras.ops.convert_to_numpy(outputs2["bounding_boxes"]["boxes"])
        classes2 = keras.ops.convert_to_numpy(
            outputs2["bounding_boxes"]["classes"]
        )

        self.assertAllClose(boxes, boxes2)
        self.assertAllClose(classes, classes2)

    def test_tf_data_compatibility(self):
        import tensorflow as tf

        layer = MaxBoundingBox(max_number=8)
        x = get_images("float32", "channels_last")
        ds = tf.data.Dataset.from_tensor_slices(x)
        ds = ds.map(
            lambda x: {
                "images": x,
                "bounding_boxes": {
                    "boxes": np.ones((4, 4)),
                    "classes": np.ones((4)),
                },
            }
        )
        ds = ds.batch(2).map(layer)
        for output in ds.take(1):
            self.assertIsInstance(output["images"], tf.Tensor)
            self.assertEqual(output["images"].shape, (2, 32, 32, 3))
            self.assertEqual(output["bounding_boxes"]["boxes"].shape, (2, 8, 4))
            self.assertEqual(output["bounding_boxes"]["classes"].shape, (2, 8))



import typing

import keras

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_bbox_format
from keras_aug._src.utils.argument_validation import standardize_data_format


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class RandomFlip(VisionRandomLayer):
    """Flip the inputs with a given probability.

    Args:
        mode: Can be `"horizontal"`, `"vertical"` and
            `"horizontal_and_vertical"`. Defaults to `"horizontal"`.
        p: A float specifying the probability. Defaults to `0.5`.
        bounding_box_format: The format of the bounding boxes. If specified,
            the available values are `"xyxy", "xywh", "center_xywh", "rel_xyxy",
            "rel_xywh", "rel_center_xywh"`. Defaults to `None`.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(
        self,
        mode: str = "horizontal",
        p: float = 0.5,
        bounding_box_format: typing.Optional[str] = None,
        data_format: typing.Optional[str] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        if not isinstance(mode, str):
            raise TypeError(
                "`mode` must be a string. "
                f"Received: mode={mode} of type {type(mode)}"
            )
        mode = mode.lower()
        if mode not in ("horizontal", "vertical", "horizontal_and_vertical"):
            raise ValueError(
                "`mode` must be either 'horizontal' or 'vertical'. "
                f"Received: mode={mode}"
            )
        self.mode = mode
        self.p = float(p)
        self.bounding_box_format = standardize_bbox_format(bounding_box_format)
        self.data_format = standardize_data_format(data_format)

        if self.data_format == "channels_last":
            self.h_axis, self.w_axis = -3, -2
        else:
            self.h_axis, self.w_axis = -2, -1

    def compute_output_shape(self, input_shape):
        return input_shape

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        random_generator = self.random_generator
        if "horizontal" in self.mode:
            p_horizontal = ops.random.uniform(
                [batch_size], seed=random_generator
            )
        else:
            p_horizontal = ops.numpy.ones([batch_size], "float32")
        if "vertical" in self.mode:
            p_vertical = ops.random.uniform([batch_size], seed=random_generator)
        else:
            p_vertical = ops.numpy.ones([batch_size], "float32")
        return dict(p_horizontal=p_horizontal, p_vertical=p_vertical)

    def augment_images(self, images, transformations, **kwargs):
        ops = self.backend
        p_horizontal = transformations["p_horizontal"]
        p_vertical = transformations["p_vertical"]
        prob_horizontal = ops.numpy.expand_dims(
            p_horizontal < self.p, axis=[1, 2, 3]
        )
        prob_vertical = ops.numpy.expand_dims(
            p_vertical < self.p, axis=[1, 2, 3]
        )

        # Horizontal
        images = ops.numpy.where(
            prob_horizontal,
            ops.numpy.flip(images, axis=self.w_axis),
            images,
        )
        # Vertical
        images = ops.numpy.where(
            prob_vertical,
            ops.numpy.flip(images, axis=self.h_axis),
            images,
        )
        return images

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(
        self,
        bounding_boxes,
        transformations,
        images=None,
        raw_images=None,
        **kwargs,
    ):
        if self.bounding_box_format is None:
            raise ValueError(
                f"{self.__class__.__name__} was called with bounding boxes,"
                "but no `bounding_box_format` was specified in the constructor."
            )
        ops = self.backend
        images_shape = ops.shape(images)
        height = images_shape[self.h_axis]
        width = images_shape[self.w_axis]
        bounding_boxes = self.bbox_backend.convert_format(
            bounding_boxes,
            source=self.bounding_box_format,
            target="xyxy",
            height=height,
            width=width,
            dtype=self.bounding_box_dtype,
        )
        x1, y1, x2, y2 = ops.numpy.split(bounding_boxes["boxes"], 4, axis=-1)

        p_horizontal = transformations["p_horizontal"]
        p_vertical = transformations["p_vertical"]
        prob_horizontal = ops.numpy.expand_dims(
            p_horizontal < self.p, axis=[1, 2]
        )
        prob_vertical = ops.numpy.expand_dims(p_vertical < self.p, axis=[1, 2])

        # Horizontal
        _x1 = ops.numpy.where(
            prob_horizontal, ops.numpy.subtract(width, x2), x1
        )
        _x2 = ops.numpy.where(
            prob_horizontal, ops.numpy.subtract(width, x1), x2
        )
        # Vertical
        _y1 = ops.numpy.where(prob_vertical, ops.numpy.subtract(width, y2), y1)
        _y2 = ops.numpy.where(prob_vertical, ops.numpy.subtract(width, y1), y2)
        outputs = ops.numpy.concatenate([_x1, _y1, _x2, _y2], axis=-1)

        bounding_boxes = bounding_boxes.copy()
        bounding_boxes["boxes"] = outputs
        bounding_boxes = self.bbox_backend.convert_format(
            bounding_boxes,
            source="xyxy",
            target=self.bounding_box_format,
            height=height,
            width=width,
            dtype=self.bounding_box_dtype,
        )
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        ops = self.backend
        p_horizontal = transformations["p_horizontal"]
        p_vertical = transformations["p_vertical"]
        prob_horizontal = ops.numpy.expand_dims(
            p_horizontal < self.p, axis=[1, 2, 3]
        )
        prob_vertical = ops.numpy.expand_dims(
            p_vertical < self.p, axis=[1, 2, 3]
        )

        # Horizontal
        segmentation_masks = ops.numpy.where(
            prob_horizontal,
            ops.numpy.flip(segmentation_masks, axis=self.w_axis),
            segmentation_masks,
        )
        # Vertical
        segmentation_masks = ops.numpy.where(
            prob_vertical,
            ops.numpy.flip(segmentation_masks, axis=self.h_axis),
            segmentation_masks,
        )
        return segmentation_masks

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "mode": self.mode,
                "p": self.p,
                "bounding_box_format": self.bounding_box_format,
            }
        )
        return config



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_aug._src.ops.bounding_box import affine
from keras_aug._src.ops.bounding_box import clip_to_images
from keras_aug._src.ops.bounding_box import convert_format
from keras_aug._src.ops.bounding_box import crop
from keras_aug._src.ops.bounding_box import pad



import keras

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class RandomChannelPermutation(VisionRandomLayer):
    """Randomly permute the channels of the input images.

    Args:
        num_channels: The number of channels to permute.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(self, num_channels: int, data_format=None, **kwargs):
        super().__init__(**kwargs)
        self.num_channels = int(num_channels)
        self.data_format = data_format or keras.config.image_data_format()

        self.channels_axis = -1 if self.data_format == "channels_last" else -3

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        random_generator = self.random_generator
        perm = ops.random.uniform(
            [batch_size, self.num_channels], seed=random_generator
        )
        perm = ops.numpy.argsort(perm, axis=-1)
        return perm

    def compute_output_shape(self, input_shape):
        images_shape, _ = self._get_shape_or_spec(input_shape)
        if images_shape[self.channels_axis] != self.num_channels:
            raise ValueError(
                "`num_channels` must match the channels of the input images. "
                f"Received: images.shape={images_shape}, "
                f"num_channels={self.num_channels}"
            )
        return input_shape

    def augment_images(self, images, transformations=None, **kwargs):
        ops = self.backend
        perm = transformations
        if self.data_format == "channels_last":
            perm = ops.numpy.expand_dims(perm, axis=[1, 2])
        else:
            perm = ops.numpy.expand_dims(perm, axis=[2, 3])
        images = ops.numpy.take_along_axis(
            images, perm, axis=self.channels_axis
        )
        return images

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(self, bounding_boxes, transformations, **kwargs):
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        return segmentation_masks

    def augment_keypoints(self, keypoints, transformations, **kwargs):
        return keypoints

    def get_config(self):
        config = super().get_config()
        config.update({"num_channels": self.num_channels})
        return config



import typing

import keras
from keras import backend

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_data_format


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class Normalize(VisionRandomLayer):
    """Normalize the images with mean and standard deviation.

    This layer will normalize each channel of the images:
    `y[c] = (x[c] - mean[c]) / std[c]`.

    Args:
        mean: Sequence of means for each channel. Defaults to
            `(0.485, 0.456, 0.406)` which is the mean values from ImageNet.
        std: Sequence of standard deviations for each channel. Defaults to
            `(0.229, 0.224, 0.225)` which is the std values from ImageNet.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(
        self,
        mean: typing.Sequence[float] = (0.485, 0.456, 0.406),
        std: typing.Sequence[float] = (0.229, 0.224, 0.225),
        data_format: typing.Optional[str] = None,
        **kwargs,
    ):
        super().__init__(has_generator=False, **kwargs)
        self.mean = tuple(mean)
        self.std = tuple(std)
        self.data_format = standardize_data_format(data_format)

        if not backend.is_float_dtype(self.compute_dtype):
            dtype = self.dtype_policy
            raise ValueError(
                "The `dtype` of Normalize must be float. "
                f"Received: dtype={dtype}"
            )

    def compute_output_shape(self, input_shape):
        return input_shape

    def augment_images(self, images, transformations, **kwargs):
        ops = self.backend
        original_dtype = backend.standardize_dtype(images.dtype)
        if self.data_format == "channels_last":
            mean = ops.numpy.expand_dims(self.mean, axis=[0, 1, 2])
            std = ops.numpy.expand_dims(self.std, axis=[0, 1, 2])
        else:
            mean = ops.numpy.expand_dims(self.mean, axis=[0, 2, 3])
            std = ops.numpy.expand_dims(self.std, axis=[0, 2, 3])
        images = ops.numpy.subtract(images, mean)
        images = ops.numpy.divide(images, std)
        return ops.cast(images, original_dtype)

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(self, bounding_boxes, transformations, **kwargs):
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        return segmentation_masks

    def augment_keypoints(self, keypoints, transformations, **kwargs):
        return keypoints

    def get_config(self):
        config = super().get_config()
        config.update({"mean": self.mean, "std": self.std})
        return config



from keras.src.utils.backend_utils import in_tf_graph

from keras_aug._src.backend.bounding_box import BoundingBoxBackend
from keras_aug._src.keras_aug_export import keras_aug_export


@keras_aug_export(parent_path=["keras_aug.ops.bounding_box"])
def convert_format(
    boxes, source: str, target: str, height=None, width=None, dtype="float32"
):
    backend = "tensorflow" if in_tf_graph() else None
    return BoundingBoxBackend(backend).convert_format(
        boxes, source, target, height=height, width=width, dtype=dtype
    )


@keras_aug_export(parent_path=["keras_aug.ops.bounding_box"])
def clip_to_images(bounding_boxes, height=None, width=None, format="xyxy"):
    backend = "tensorflow" if in_tf_graph() else None
    return BoundingBoxBackend(backend).clip_to_images(
        bounding_boxes, height=height, width=width, format=format
    )


@keras_aug_export(parent_path=["keras_aug.ops.bounding_box"])
def affine(
    boxes,
    angle,
    translate_x,
    translate_y,
    scale,
    shear_x,
    shear_y,
    height,
    width,
    center_x=None,
    center_y=None,
    format="xyxy",
):
    if format != "xyxy":
        raise NotImplementedError
    backend = "tensorflow" if in_tf_graph() else None
    return BoundingBoxBackend(backend).affine(
        boxes,
        angle,
        translate_x,
        translate_y,
        scale,
        shear_x,
        shear_y,
        height,
        width,
        center_x=center_x,
        center_y=center_y,
    )


@keras_aug_export(parent_path=["keras_aug.ops.bounding_box"])
def crop(boxes, top, left, height, width, format="xyxy"):
    if format != "xyxy":
        raise NotImplementedError
    backend = "tensorflow" if in_tf_graph() else None
    return BoundingBoxBackend(backend).crop(boxes, top, left, height, width)


@keras_aug_export(parent_path=["keras_aug.ops.bounding_box"])
def pad(boxes, top, left, format="xyxy"):
    if format != "xyxy":
        raise NotImplementedError
    backend = "tensorflow" if in_tf_graph() else None
    return BoundingBoxBackend(backend).pad(boxes, top, left)



import typing
from collections.abc import Sequence

import keras
from keras import KerasTensor
from keras import backend

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_bbox_format
from keras_aug._src.utils.argument_validation import standardize_data_format


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class Resize(VisionRandomLayer):
    """Resize the inputs to the given size.

    Args:
        size: Desired output size. If `size` is a sequence like `(h, w)`,
            output size will be matched to this. If `size` is an int, smaller
            edge of the image will be matched to this number.
            i.e, if height > width, then image will be rescaled to
            (size * height / width, size).
        interpolation: The interpolation mode. Available values are:
            `"nearest", "bilinear", "bicubic"`. Defaults to `"bilinear"`.
        antialias: Whether to apply antialiasing. It only affects bilinear and
            bicubic modes. Defaults to `True`.
        along_long_edge: Whether to resize the long edge instead of the short
            edge. It only affects when `size` is a single integer. Defaults to
            `False`.
        bounding_box_format: The format of the bounding boxes. If specified,
            the available values are `"xyxy", "xywh", "center_xywh", "rel_xyxy",
            "rel_xywh", "rel_center_xywh"`. Defaults to `None`.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(
        self,
        size: typing.Union[typing.Sequence[int], int],
        interpolation: str = "bilinear",
        antialias: bool = True,
        along_long_edge: bool = False,
        bounding_box_format: typing.Optional[str] = None,
        data_format: typing.Optional[str] = None,
        **kwargs,
    ):
        super().__init__(has_generator=False, **kwargs)
        # Check
        if not isinstance(size, (int, Sequence)):
            raise ValueError(
                "`size` must be an integer or a sequence. "
                f"Received: size={size} (of type {type(size)})"
            )
        if isinstance(size, Sequence):
            if len(size) not in (1, 2):
                raise ValueError(
                    "If `size` is a sequence, the length must be 2. "
                    f"Received: size={size}"
                )
        if along_long_edge and not isinstance(size, int):
            raise ValueError(
                "If `along_long_edge=True`, `size` must be a single integer. "
                f"Received: size={size}"
            )

        self.size = size
        self.interpolation = interpolation
        self.antialias = antialias
        self.along_long_edge = along_long_edge
        self.bounding_box_format = standardize_bbox_format(bounding_box_format)
        self.data_format = standardize_data_format(data_format)

        if self.data_format == "channels_last":
            self.h_axis, self.w_axis = -3, -2
        else:
            self.h_axis, self.w_axis = -2, -1

    def compute_output_spec(self, inputs):
        images, segmentation_masks = self._get_shape_or_spec(inputs)
        images_shape = list(images.shape)
        h, w = images_shape[self.h_axis], images_shape[self.w_axis]

        # Compute outputs
        if h is not None and w is not None:
            new_h, new_w = self._compute_resized_output_size(
                h, w, self.size, self.along_long_edge
            )
        else:
            new_h, new_w = None, None
        images_shape = list(images_shape)
        images_shape[self.h_axis] = new_h
        images_shape[self.w_axis] = new_w
        images_shape = list(
            int(s) if s is not None else None for s in images_shape
        )
        images = KerasTensor(images_shape, dtype=images.dtype)
        if segmentation_masks is not None:
            segmentation_masks_shape = list(segmentation_masks)
            segmentation_masks_shape[self.h_axis] = new_h
            segmentation_masks_shape[self.w_axis] = new_w
            segmentation_masks_shape = list(
                int(s) if s is not None else None
                for s in segmentation_masks_shape
            )
            segmentation_masks = KerasTensor(
                segmentation_masks_shape, dtype=segmentation_masks.dtype
            )

        return self._set_spec(inputs, images, segmentation_masks)

    def compute_output_shape(self, input_shape):
        images_shape, segmentation_masks_shape = self._get_shape_or_spec(
            input_shape
        )
        h, w = images_shape[self.h_axis], images_shape[self.w_axis]

        # Compute output shape
        if h is not None and w is not None:
            new_h, new_w = self._compute_resized_output_size(
                h, w, self.size, self.along_long_edge
            )
        else:
            new_h, new_w = None, None
        images_shape = list(images_shape)
        images_shape[self.h_axis] = new_h
        images_shape[self.w_axis] = new_w
        if segmentation_masks_shape is not None:
            segmentation_masks_shape = list(segmentation_masks_shape)
            segmentation_masks_shape[self.h_axis] = new_h
            segmentation_masks_shape[self.w_axis] = new_w

        return self._set_shape(
            input_shape, images_shape, segmentation_masks_shape
        )

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        images_shape = ops.shape(images)
        h, w = images_shape[self.h_axis], images_shape[self.w_axis]
        new_h, new_w = self._compute_resized_output_size(
            h, w, self.size, self.along_long_edge
        )
        output_size = ops.numpy.stack([new_h, new_w])
        return output_size

    def augment_images(self, images, transformations, **kwargs):
        ops = self.backend
        original_dtype = backend.standardize_dtype(images.dtype)
        output_size = transformations
        images = ops.image.resize(
            images,
            (output_size[0], output_size[1]),
            interpolation=self.interpolation,
            antialias=self.antialias,
            data_format=self.data_format,
        )
        return ops.cast(images, original_dtype)

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(
        self,
        bounding_boxes,
        transformations,
        images=None,
        raw_images=None,
        **kwargs,
    ):
        if self.bounding_box_format is None:
            raise ValueError(
                f"{self.__class__.__name__} was called with bounding boxes,"
                "but no `bounding_box_format` was specified in the constructor."
            )
        ops = self.backend

        ori_images_shape = ops.shape(raw_images)
        ori_height = ori_images_shape[self.h_axis]
        ori_width = ori_images_shape[self.w_axis]
        output_size = transformations

        # Resize
        bounding_boxes = self.bbox_backend.convert_format(
            bounding_boxes,
            source=self.bounding_box_format,
            target="rel_xyxy",
            height=ori_height,
            width=ori_width,
            dtype=self.bounding_box_dtype,
        )
        bounding_boxes = self.bbox_backend.convert_format(
            bounding_boxes,
            source="rel_xyxy",
            target=self.bounding_box_format,
            height=ops.cast(output_size[0], "float32"),
            width=ops.cast(output_size[1], "float32"),
            dtype=self.bounding_box_dtype,
        )
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        ops = self.backend
        original_dtype = backend.standardize_dtype(segmentation_masks.dtype)
        output_size = transformations
        segmentation_masks = ops.image.resize(
            segmentation_masks,
            (output_size[0], output_size[1]),
            interpolation="nearest",
            antialias=self.antialias,
            data_format=self.data_format,
        )
        return ops.cast(segmentation_masks, original_dtype)

    def _compute_resized_output_size(
        self, height, width, size, along_long_edge=False
    ):
        ops = self.backend
        if isinstance(size, int):
            size = [size]
        if len(size) == 1 and along_long_edge is False:
            new_short = size[0]
            new_long = ops.cast(
                ops.numpy.where(
                    height < width,
                    new_short * width / height,
                    new_short * height / width,
                ),
                "int32",
            )
            new_h = ops.numpy.where(height < width, new_short, new_long)
            new_w = ops.numpy.where(width < height, new_short, new_long)
        elif len(size) == 1 and along_long_edge is True:
            new_long = size[0]
            new_short = ops.cast(
                ops.numpy.where(
                    height > width,
                    new_long * width / height,
                    new_long * height / width,
                ),
                "int32",
            )
            new_h = ops.numpy.where(height < width, new_short, new_long)
            new_w = ops.numpy.where(width < height, new_short, new_long)
        elif len(size) == 2:
            new_h, new_w = size
        else:
            raise ValueError
        return new_h, new_w

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "size": self.size,
                "interpolation": self.interpolation,
                "antialias": self.antialias,
                "along_long_edge": self.along_long_edge,
                "bounding_box_format": self.bounding_box_format,
            }
        )
        return config



import math
import typing

import keras
from keras import backend

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_bbox_format
from keras_aug._src.utils.argument_validation import standardize_data_format
from keras_aug._src.utils.argument_validation import standardize_interpolation
from keras_aug._src.utils.argument_validation import standardize_size
from keras_aug._src.utils.argument_validation import standardize_value_range


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class RandomResizedCrop(VisionRandomLayer):
    """Crop a random portion of the inputs and resize it to a given size.

    Args:
        size: Desired output size. If `size` is a sequence like `(h, w)`,
            output size will be matched to this. If `size` is an int,  a square
            output size `(size, size)` is made.
        scale: A tuple of floats specifying the lower and upper bounds for the
            random area of the crop, before resizing. The scale is defined with
            respect to the area of the original image.
        ratio: A tuple of floats specifying the lower and upper bounds for the
            random aspect ratio of the crop, before resizing.
        interpolation: The interpolation mode. Available values are:
            `"nearest", "bilinear", "bicubic"`. Defaults to `"bilinear"`.
        antialias: Whether to apply antialiasing. It only affects bilinear and
            bicubic modes. Defaults to `True`.
        along_long_edge: Whether to resize the long edge instead of the short
            edge. It only affects when `size` is a single integer. Defaults to
            `False`.
        bounding_box_format: The format of the bounding boxes. If specified,
            the available values are `"xyxy", "xywh", "center_xywh", "rel_xyxy",
            "rel_xywh", "rel_center_xywh"`. Defaults to `None`.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(
        self,
        size: typing.Union[typing.Sequence[int], int],
        scale: typing.Tuple[float, float] = (0.08, 1.0),
        ratio: typing.Tuple[float, float] = (3 / 4, 4 / 3),
        interpolation: str = "bilinear",
        antialias: bool = True,
        bounding_box_format: typing.Optional[str] = None,
        data_format: typing.Optional[str] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.size = standardize_size(size)
        self.scale = standardize_value_range(scale)
        self.ratio = standardize_value_range(ratio)
        self.interpolation = standardize_interpolation(interpolation)
        self.antialias = antialias
        self.bounding_box_format = standardize_bbox_format(bounding_box_format)
        self.data_format = standardize_data_format(data_format)

        self.log_ratio = (math.log(self.ratio[0]), math.log(self.ratio[1]))
        if self.data_format == "channels_last":
            self.h_axis, self.w_axis = -3, -2
        else:
            self.h_axis, self.w_axis = -2, -1

    def compute_output_shape(self, input_shape):
        images_shape, segmentation_masks_shape = self._get_shape_or_spec(
            input_shape
        )
        images_shape = list(images_shape)
        images_shape[self.h_axis] = self.size[0]
        images_shape[self.w_axis] = self.size[1]
        if segmentation_masks_shape is not None:
            segmentation_masks_shape = list(segmentation_masks_shape)
            segmentation_masks_shape[self.h_axis] = self.size[0]
            segmentation_masks_shape[self.w_axis] = self.size[1]
        return self._set_shape(
            input_shape, images_shape, segmentation_masks_shape
        )

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        random_generator = self.random_generator
        images_shape = ops.shape(images)
        height, width = images_shape[self.h_axis], images_shape[self.w_axis]

        area = ops.cast(height * width, "float32")
        scale = ops.random.uniform(
            shape=(),
            minval=self.scale[0],
            maxval=self.scale[1],
            seed=random_generator,
        )
        target_area = area * scale
        log_ratio = ops.random.uniform(
            shape=(),
            minval=self.log_ratio[0],
            maxval=self.log_ratio[1],
            seed=random_generator,
        )
        aspect_ratio = ops.numpy.exp(log_ratio)

        w = ops.cast(
            ops.numpy.round(ops.numpy.sqrt(target_area * aspect_ratio)), "int32"
        )
        h = ops.cast(
            ops.numpy.round(ops.numpy.sqrt(target_area / aspect_ratio)), "int32"
        )
        w = ops.numpy.clip(w, 1, width)
        h = ops.numpy.clip(h, 1, height)
        i = ops.random.randint(
            shape=(), minval=0, maxval=(height - h + 1), seed=random_generator
        )
        j = ops.random.randint(
            shape=(), minval=0, maxval=(width - w + 1), seed=random_generator
        )
        return dict(top=i, left=j, height=h, width=w)

    def augment_images(self, images, transformations, **kwargs):
        ops = self.backend
        original_dtype = backend.standardize_dtype(images.dtype)
        top = transformations["top"]
        left = transformations["left"]
        height = transformations["height"]
        width = transformations["width"]

        # Crop
        images = self.image_backend.crop(
            images, top, left, height, width, self.data_format
        )

        # Resize
        images = ops.image.resize(
            images,
            (self.size[0], self.size[1]),
            interpolation=self.interpolation,
            antialias=self.antialias,
            data_format=self.data_format,
        )
        return ops.cast(images, original_dtype)

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(
        self,
        bounding_boxes,
        transformations,
        images=None,
        raw_images=None,
        **kwargs,
    ):
        if self.bounding_box_format is None:
            raise ValueError(
                f"{self.__class__.__name__} was called with bounding boxes,"
                "but no `bounding_box_format` was specified in the constructor."
            )
        ops = self.backend

        ori_images_shape = ops.shape(raw_images)
        ori_height = ori_images_shape[self.h_axis]
        ori_width = ori_images_shape[self.w_axis]
        top = ops.cast(transformations["top"], "float32")
        left = ops.cast(transformations["left"], "float32")
        h_scale = self.size[0] / ops.cast(transformations["height"], "float32")
        w_scale = self.size[1] / ops.cast(transformations["width"], "float32")
        bounding_boxes = self.bbox_backend.convert_format(
            bounding_boxes,
            source=self.bounding_box_format,
            target="xyxy",
            height=ori_height,
            width=ori_width,
            dtype=self.bounding_box_dtype,
        )

        x1, y1, x2, y2 = ops.numpy.split(bounding_boxes["boxes"], 4, axis=-1)
        x1 = (x1 - left) * w_scale
        y1 = (y1 - top) * h_scale
        x2 = (x2 - left) * w_scale
        y2 = (y2 - top) * h_scale
        outputs = ops.numpy.concatenate([x1, y1, x2, y2], axis=-1)

        bounding_boxes = bounding_boxes.copy()
        bounding_boxes["boxes"] = outputs
        bounding_boxes = self.bbox_backend.clip_to_images(
            bounding_boxes,
            height=self.size[0],
            width=self.size[1],
            format="xyxy",
        )
        bounding_boxes = self.bbox_backend.convert_format(
            bounding_boxes,
            source="xyxy",
            target=self.bounding_box_format,
            height=self.size[0],
            width=self.size[1],
            dtype=self.bounding_box_dtype,
        )
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        ops = self.backend
        original_dtype = backend.standardize_dtype(segmentation_masks.dtype)
        top = transformations["top"]
        left = transformations["left"]
        height = transformations["height"]
        width = transformations["width"]

        # Crop
        segmentation_masks = self.image_backend.crop(
            segmentation_masks, top, left, height, width, self.data_format
        )

        # Resize
        segmentation_masks = ops.image.resize(
            segmentation_masks,
            (self.size[0], self.size[1]),
            interpolation="nearest",
            antialias=False,
            data_format=self.data_format,
        )
        return ops.cast(segmentation_masks, original_dtype)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "size": self.size,
                "scale": self.scale,
                "ratio": self.ratio,
                "interpolation": self.interpolation,
                "antialias": self.antialias,
                "bounding_box_format": self.bounding_box_format,
            }
        )
        return config



import typing

import keras
from keras import backend
from keras import saving
from keras.src.utils.backend_utils import in_tf_graph

from keras_aug._src.backend.dynamic_backend import DynamicBackend
from keras_aug._src.backend.dynamic_backend import DynamicRandomGenerator
from keras_aug._src.keras_aug_export import keras_aug_export


@keras_aug_export(parent_path=["keras_aug.layers.composition"])
@keras.saving.register_keras_serializable(package="keras_aug")
class RandomOrder(keras.Layer):
    """Apply a list of transformations in a random order.

    Note that due to implementation limitations, the randomness occurs in a
    batch manner.

    Args:
        transforms: A list of transformations or a `keras.Layer`.
    """

    def __init__(self, transforms, seed=None, **kwargs):
        super().__init__(**kwargs)
        self._backend = DynamicBackend(backend.backend())
        self._random_generator = DynamicRandomGenerator(
            backend.backend(), seed=seed
        )
        self.seed = seed

        # Check
        if not isinstance(transforms, (typing.Sequence, keras.Layer)):
            raise ValueError(
                "`transforms` must be a sequence (e.g. tuple and list) or a "
                "`keras.Layer`. "
                f"Received: transforms={transforms} of type {type(transforms)}"
            )
        if isinstance(transforms, keras.Layer):
            transforms = [transforms]
        self.transforms = list(transforms)
        self.total = len(self.transforms)

        self._convert_input_args = False
        self._allow_non_tensor_positional_args = True
        self.autocast = False

    @property
    def backend(self):
        return self._backend.backend

    @property
    def random_generator(self):
        return self._random_generator.random_generator

    def compute_output_shape(self, input_shape):
        output_shape = input_shape
        for transfrom in self.transforms:
            output_shape = transfrom.compute_output_shape(output_shape)
        return output_shape

    def get_params(self):
        ops = self.backend
        random_generator = self.random_generator

        fn_idx = ops.random.shuffle(
            ops.numpy.arange(self.total, dtype="int32"), seed=random_generator
        )
        return fn_idx

    def __call__(self, inputs, **kwargs):
        if in_tf_graph():
            self._set_backend("tensorflow")
            try:
                outputs = super().__call__(inputs, **kwargs)
            finally:
                self._reset_backend()
            return outputs
        else:
            return super().__call__(inputs, **kwargs)

    def call(self, inputs):
        ops = self.backend
        fn_idx = self.get_params()

        outputs = inputs
        for i in range(self.total):
            idx = fn_idx[i]
            outputs = ops.core.switch(idx, self.transforms, outputs)
        return outputs

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "transforms": saving.serialize_keras_object(self.transforms),
                "seed": self.seed,
            }
        )
        return config

    @classmethod
    def from_config(cls, config, custom_objects=None):
        config = config.copy()
        config["transforms"] = saving.deserialize_keras_object(
            config["transforms"], custom_objects=custom_objects
        )
        return cls(**config)

    def _set_backend(self, name):
        self._backend.set_backend(name)
        self._random_generator.set_generator(name)

    def _reset_backend(self):
        self._backend.reset()
        self._random_generator.reset()



import math
import numbers
import typing
import warnings
from collections.abc import Sequence

import keras
from keras import backend

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_data_format
from keras_aug._src.utils.argument_validation import standardize_value_range


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class RandomErasing(VisionRandomLayer):
    """Randomly select a rectangle region in the input images erase its pixels.

    If image size is smaller than output size along any edge, image is padded
    with `padding_value` and then cropped.

    References:
    - [Random Erasing Data Augmentation](https://arxiv.org/abs/1708.04896)

    Args:
        p: A float specifying the probability. Defaults to `0.5`.
        scale: A tuple of floats specifying the lower and upper bounds for the
            range of proportion of erased area against input images. Defaults to
            `(0.02, 0.33)`.
        ratio: A tuple of floats specifying the lower and upper bounds for the
            range of aspect ratio of erased area. Defaults to `(0.3, 3.3)`.
        value: The erasing values. If a single float, it is used to erase all
            pixels. If a tuple of floats, it is used to erase each channels
            respectively. If a str of `"random", erasing each pixel with random
            values. `Defaults to `0`.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(
        self,
        p: float = 0.5,
        scale: typing.Sequence[float] = (0.02, 0.33),
        ratio: typing.Sequence[float] = (0.3, 3.3),
        value: typing.Union[float, typing.Sequence[float], str] = 0.0,
        data_format: typing.Optional[str] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        # Check
        if isinstance(value, str) and value != "random":
            raise ValueError(
                "If `value` is a string, it must be `'random'`. "
                f"Received: value={value}"
            )
        if isinstance(value, numbers.Number):
            value = float(value)
        if not isinstance(value, str) and isinstance(value, Sequence):
            value = tuple(value)
        self.p = float(p)
        self.scale = standardize_value_range(scale)
        self.ratio = standardize_value_range(ratio)
        self.value = value
        self.data_format = standardize_data_format(data_format)

        self.log_ratio = (math.log(self.ratio[0]), math.log(self.ratio[1]))
        if self.data_format == "channels_last":
            self.h_axis, self.w_axis, self.c_axis = -3, -2, -1
        else:
            self.c_axis, self.h_axis, self.w_axis = -3, -2, -1
        if self.value == "random" and backend.is_int_dtype(self.compute_dtype):
            raise ValueError(
                "If `value` is 'random', the compute dtype of this layer "
                "should be float. "
                f"Received: value={value}, compute_dtype={self.compute_dtype}"
            )

    def compute_output_shape(self, input_shape):
        if isinstance(input_shape, dict):
            images_shape = input_shape[self.IMAGES]
        else:
            images_shape = input_shape
        channels = images_shape[self.c_axis]
        if isinstance(self.value, tuple) and len(self.value) != channels:
            raise ValueError(
                "If `value` is a sequence, the length of `value` must be the "
                "same as channels of input images. "
                f"Received: value={self.value}, images.shape={images_shape}"
            )
        return input_shape

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        random_generator = self.random_generator
        images_shape = ops.shape(images)
        height, width = images_shape[self.h_axis], images_shape[self.w_axis]

        area = ops.cast(height * width, "float32")
        scale = ops.random.uniform(
            shape=[batch_size],
            minval=self.scale[0],
            maxval=self.scale[1],
            seed=random_generator,
        )
        target_area = area * scale
        log_ratio = ops.random.uniform(
            shape=[batch_size],
            minval=self.log_ratio[0],
            maxval=self.log_ratio[1],
            seed=random_generator,
        )
        aspect_ratio = ops.numpy.exp(log_ratio)

        w = ops.cast(
            ops.numpy.round(ops.numpy.sqrt(target_area * aspect_ratio)), "int32"
        )
        h = ops.cast(
            ops.numpy.round(ops.numpy.sqrt(target_area / aspect_ratio)), "int32"
        )
        w = ops.numpy.clip(w, 1, width)
        h = ops.numpy.clip(h, 1, height)
        i = ops.random.uniform(shape=[batch_size], seed=random_generator)
        i = ops.cast(
            ops.numpy.round(ops.numpy.multiply(i, (height - h + 1))), "int32"
        )
        j = ops.random.uniform(shape=[batch_size], seed=random_generator)
        j = ops.cast(
            ops.numpy.round(ops.numpy.multiply(j, (width - w + 1))), "int32"
        )

        # Get values
        if isinstance(self.value, str):
            dtype = backend.result_type(images.dtype, float)
            v = ops.random.normal(
                ops.shape(images), dtype=dtype, seed=random_generator
            )
        elif isinstance(self.value, float):
            dtype = backend.standardize_dtype(images.dtype)
            v = ops.numpy.full(ops.shape(images), self.value, dtype=dtype)
        elif isinstance(self.value, tuple):
            dtype = backend.standardize_dtype(images.dtype)
            v = ops.convert_to_tensor(self.value, dtype=dtype)  # [c]
            if self.data_format == "channels_last":
                v = ops.numpy.expand_dims(v, axis=[0, 1, 2])
                v = ops.numpy.tile(v, [batch_size, height, width, 1])
            else:
                v = ops.numpy.expand_dims(v, axis=[0, -1, -2])
                v = ops.numpy.tile(v, [batch_size, 1, height, width])
        return dict(top=i, left=j, height=h, width=w, value=v)

    def augment_images(self, images, transformations, **kwargs):
        left = transformations["left"]
        top = transformations["top"]
        width = transformations["width"]
        height = transformations["height"]
        right = left + width
        bottom = top + height
        value = transformations["value"]
        images = self.image_backend.fill_rectangles(
            images,
            value,
            (left, top, right, bottom),
            data_format=self.data_format,
        )
        return images

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(
        self,
        bounding_boxes,
        transformations,
        images=None,
        raw_images=None,
        **kwargs,
    ):
        warnings.warn(
            f"{self.__class__.__name__} is currently passing through "
            "`bounding_boxes`. This will likely change in the future."
        )
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        warnings.warn(
            f"{self.__class__.__name__} is currently passing through "
            "`segmentation_masks`. This will likely change in the future."
        )
        return segmentation_masks

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "p": self.p,
                "scale": self.scale,
                "ratio": self.ratio,
                "value": self.value,
            }
        )
        return config



import keras
import numpy as np
from absl.testing import parameterized
from keras import backend
from keras.src import testing
from keras.src.testing.test_utils import named_product

from keras_aug._src.layers.vision.random_rotation import RandomRotation
from keras_aug._src.utils.test_utils import get_images


class FixedRandomRotation(RandomRotation):
    def get_params(self, batch_size, images=None, **kwargs):
        if batch_size == 1:
            return dict(angle=keras.ops.array([10.0]))
        elif batch_size == 2:
            return dict(angle=keras.ops.array([10.0, 10.0]))


class RandomRotationTest(testing.TestCase, parameterized.TestCase):
    regular_args = dict(degree=[-10, 10])

    def setUp(self):
        # Defaults to channels_last
        self.data_format = backend.image_data_format()
        backend.set_image_data_format("channels_last")
        return super().setUp()

    def tearDown(self) -> None:
        backend.set_image_data_format(self.data_format)
        return super().tearDown()

    @parameterized.named_parameters(
        named_product(
            interpolation=["nearest", "bilinear"], dtype=["float32", "uint8"]
        )
    )
    def test_correctness(self, interpolation, dtype):
        # Test channels_last
        np.random.seed(42)
        x = get_images(dtype, "channels_last")
        layer = FixedRandomRotation(
            interpolation=interpolation,
            data_format="channels_last",
            dtype=dtype,
        )
        y = layer(x)

        # TODO: It is difficult to be consistent with `TF.affine`
        self.assertEqual(y.shape, (2, 32, 32, 3))
        self.assertDType(y, dtype)
        self.assertNotAllClose(y, x)

        # Test channels_first
        backend.set_image_data_format("channels_first")
        np.random.seed(42)
        x = get_images(dtype, "channels_first")
        layer = FixedRandomRotation(
            interpolation=interpolation,
            data_format="channels_first",
            dtype=dtype,
        )
        y = layer(x)

        # TODO: It is difficult to be consistent with `TF.affine`
        self.assertEqual(y.shape, (2, 3, 32, 32))
        self.assertDType(y, dtype)
        self.assertNotAllClose(y, x)

    def test_shape(self):
        # Test dynamic shape
        x = keras.KerasTensor((None, None, None, 3))
        y = RandomRotation(**self.regular_args)(x)
        self.assertEqual(y.shape, (None, None, None, 3))

        # Test static shape
        x = keras.KerasTensor((None, 32, 32, 3))
        y = RandomRotation(**self.regular_args)(x)
        self.assertEqual(y.shape, (None, 32, 32, 3))

    def test_model(self):
        # Test dynamic shape
        layer = RandomRotation(**self.regular_args)
        inputs = keras.layers.Input(shape=[None, None, 3])
        outputs = layer(inputs)
        model = keras.models.Model(inputs, outputs)
        self.assertEqual(model.output_shape, (None, None, None, 3))

        # Test static shape
        layer = RandomRotation(**self.regular_args)
        inputs = keras.layers.Input(shape=[32, 32, 3])
        outputs = layer(inputs)
        model = keras.models.Model(inputs, outputs)
        self.assertEqual(model.output_shape, (None, 32, 32, 3))

    def test_config(self):
        x = get_images("float32", "channels_last")
        layer = RandomRotation(degree=[10, 10])
        y = layer(x)

        layer = RandomRotation.from_config(layer.get_config())
        y2 = layer(x)
        self.assertAllClose(y, y2)

    def test_tf_data_compatibility(self):
        import tensorflow as tf

        layer = RandomRotation(**self.regular_args)
        x = get_images("float32", "channels_last")
        ds = tf.data.Dataset.from_tensor_slices(x).batch(2).map(layer)
        for output in ds.take(1):
            self.assertIsInstance(output, tf.Tensor)
            self.assertEqual(output.shape, (2, 32, 32, 3))

    def test_augment_bounding_box(self):
        # Test full bounding boxes
        images = np.zeros([1, 32, 32, 3]).astype("float32")
        boxes = {
            "boxes": np.array(
                [
                    [[0, 0, 1, 1], [0, 0, 1, 1], [0, 0, 1, 1], [0, 0, 1, 1]],
                ],
                "float32",
            ),
            "classes": np.array([[0, 0, 0, 0]], "float32"),
        }
        input = {"images": images, "bounding_boxes": boxes}
        layer = FixedRandomRotation(bounding_box_format="rel_xyxy")

        output = layer(input)
        self.assertAllClose(output["bounding_boxes"]["boxes"], boxes["boxes"])
        self.assertAllClose(
            output["bounding_boxes"]["classes"], boxes["classes"]
        )

        # Test arbitrary bounding boxes
        images = np.zeros([1, 32, 32, 3]).astype("float32")
        boxes = {
            "boxes": np.array(
                [
                    [
                        [3, 4, 17, 18],
                        [10, 12, 16, 19],
                        [0, 0, 1, 1],
                        [15, 12, 17, 20],
                    ],
                ],
                "float32",
            ),
            "classes": np.array([[0, 1, 2, 3]], "float32"),
        }
        input = {"images": images, "bounding_boxes": boxes}
        layer = FixedRandomRotation(bounding_box_format="xyxy")

        output = layer(input)
        expected_boxes = {
            "boxes": np.array(
                [
                    [
                        [2.850203, 1.924881, 19.068584, 18.143265],
                        [9.57021, 11.01888, 16.694592, 18.954426],
                        [2.847799, 0.0, 4.006255, 0.0],
                        [14.3206, 11.887121, 17.679401, 20.11288],
                    ],
                ],
                "float32",
            ),
            "classes": np.array([[0, 1, -1, 3]], "float32"),
        }
        self.assertAllClose(
            output["bounding_boxes"]["boxes"], expected_boxes["boxes"]
        )
        self.assertAllClose(
            output["bounding_boxes"]["classes"], expected_boxes["classes"]
        )

    def test_augment_segmentation_mask(self):
        num_classes = 8
        images_shape = (2, 32, 32, 3)
        masks_shape = (2, 32, 32, 1)
        images = np.random.uniform(size=images_shape).astype("float32")
        masks = np.random.randint(2, size=masks_shape) * (num_classes - 1)
        inputs = {"images": images, "segmentation_masks": masks}

        layer = FixedRandomRotation()
        output = layer(inputs)
        self.assertEqual(output["segmentation_masks"].shape, masks_shape)
        # TODO: Verify the correctness



import keras
import numpy as np
from absl.testing import parameterized
from keras import backend
from keras.src import testing
from keras.src.testing.test_utils import named_product

from keras_aug._src.layers.vision.random_erasing import RandomErasing
from keras_aug._src.utils.test_utils import get_images


class FixedRandomErasing(RandomErasing):
    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        images_shape = ops.shape(images)
        height, width = images_shape[self.h_axis], images_shape[self.w_axis]
        top = ops.numpy.zeros([batch_size])
        left = ops.numpy.zeros([batch_size])
        h = ops.numpy.ones([batch_size]) * 10
        w = ops.numpy.ones([batch_size]) * 10
        if isinstance(self.value, str):
            dtype = backend.result_type(images.dtype, float)
            v = ops.random.normal(ops.shape(images), dtype=dtype)
        elif isinstance(self.value, float):
            dtype = backend.standardize_dtype(images.dtype)
            v = ops.numpy.full(ops.shape(images), self.value)
            v = ops.cast(v, dtype)
        elif isinstance(self.value, tuple):
            dtype = backend.standardize_dtype(images.dtype)
            v = ops.convert_to_tensor(self.value)  # [c]
            v = ops.cast(v, dtype)
            if self.data_format == "channels_last":
                v = ops.numpy.expand_dims(v, axis=[0, 1, 2])
                v = ops.numpy.tile(v, [batch_size, height, width, 1])
            else:
                v = ops.numpy.expand_dims(v, axis=[0, -1, -2])
                v = ops.numpy.tile(v, [batch_size, 1, height, width])
        return dict(top=top, left=left, height=h, width=w, value=v)


class RandomErasingTest(testing.TestCase, parameterized.TestCase):
    def setUp(self):
        # Defaults to channels_last
        self.data_format = backend.image_data_format()
        backend.set_image_data_format("channels_last")
        return super().setUp()

    def tearDown(self) -> None:
        backend.set_image_data_format(self.data_format)
        return super().tearDown()

    @parameterized.named_parameters(
        named_product(
            value=[0.0, (1.0, 1.0, 1.0), "random"], dtype=["float32", "uint8"]
        )
    )
    def test_correctness(self, value, dtype):
        if dtype == "uint8" and value == "random":
            self.skipTest("value='random' doesn't support dtype='uint8'")

        # Test channels_last
        images = get_images(dtype, "channels_last")
        layer = FixedRandomErasing(value=value, dtype=dtype)
        outputs = layer(images)

        self.assertDType(outputs, dtype)
        if value == 0.0:
            self.assertAllClose(
                outputs[:, 0:10, 0:10, :],
                np.zeros_like(outputs[:, 0:10, 0:10, :]),
            )
        elif value == (1.0, 1.0, 1.0):
            self.assertAllClose(
                outputs[:, 0:10, 0:10, :],
                np.ones_like(outputs[:, 0:10, 0:10, :]),
            )
        else:
            pass
        self.assertAllClose(outputs[:, 10:, 10:, :], images[:, 10:, 10:, :])

        # Test channels_first
        backend.set_image_data_format("channels_first")
        images = get_images(dtype, "channels_first")
        layer = FixedRandomErasing(value=value, dtype=dtype)
        outputs = layer(images)

        self.assertDType(outputs, dtype)
        if value == 0.0:
            self.assertAllClose(
                outputs[:, :, 0:10, 0:10],
                np.zeros_like(outputs[:, :, 0:10, 0:10]),
            )
        elif value == (1.0, 1.0, 1.0):
            self.assertAllClose(
                outputs[:, :, 0:10, 0:10],
                np.ones_like(outputs[:, :, 0:10, 0:10]),
            )
        else:
            pass
        self.assertAllClose(outputs[:, :, 10:, 10:], images[:, :, 10:, 10:])

    def test_shape(self):
        # Test dynamic shape
        x = keras.KerasTensor((None, None, None, 3))
        y = RandomErasing()(x)
        self.assertEqual(y.shape, (None, None, None, 3))

        # Test static shape
        x = keras.KerasTensor((None, 32, 32, 3))
        y = RandomErasing()(x)
        self.assertEqual(y.shape, (None, 32, 32, 3))

    def test_model(self):
        layer = RandomErasing()
        inputs = keras.layers.Input(shape=(None, None, 3))
        outputs = layer(inputs)
        model = keras.models.Model(inputs, outputs)
        self.assertEqual(model.output_shape, (None, None, None, 3))

    def test_config(self):
        x = get_images("float32", "channels_last")
        layer = FixedRandomErasing()
        y = layer(x)

        layer = FixedRandomErasing.from_config(layer.get_config())
        y2 = layer(x)
        self.assertAllClose(y, y2)

    def test_tf_data_compatibility(self):
        import tensorflow as tf

        layer = RandomErasing()
        x = get_images("float32", "channels_last")
        ds = tf.data.Dataset.from_tensor_slices(x).batch(2).map(layer)
        for output in ds.take(1):
            self.assertIsInstance(output, tf.Tensor)
            self.assertEqual(output.shape, (2, 32, 32, 3))



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_aug import layers
from keras_aug import ops
from keras_aug import visualization
from keras_aug._src.version import version

__version__ = "1.0.0"



import keras
from keras import backend

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_data_format


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class RandomEqualize(VisionRandomLayer):
    """Equalize the histogram of the images randomly with a given probability.

    This class equalizes the histogram of the images by applying a non-linear
    mapping in order to create a uniform distribution of grayscale values in
    the outputs.

    Args:
        bins: The number of bins to use in histogram equalization. The value
            must be in the range of `[0, 256]`. Defaults to `256`.
        p: A float specifying the probability. Defaults to `0.5`.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(self, bins=256, p: float = 0.5, data_format=None, **kwargs):
        super().__init__(**kwargs)
        self.bins = bins
        self.p = float(p)
        self.data_format = standardize_data_format(data_format)

    def compute_output_shape(self, input_shape):
        return input_shape

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        random_generator = self.random_generator
        p = ops.random.uniform([batch_size], seed=random_generator)
        return p

    def augment_images(self, images, transformations=None, **kwargs):
        ops = self.backend
        p = transformations

        def equalize(images):
            original_dtype = backend.standardize_dtype(images.dtype)
            images = self.image_backend.transform_dtype(
                images, images.dtype, "uint8"
            )
            images = self.image_backend.equalize(
                images, self.bins, self.data_format
            )
            images = self.image_backend.transform_dtype(
                images, images.dtype, original_dtype
            )
            return images

        prob = ops.numpy.expand_dims(p < self.p, axis=[1, 2, 3])
        images = ops.numpy.where(prob, equalize(images), images)
        return images

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(self, bounding_boxes, transformations, **kwargs):
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        return segmentation_masks

    def augment_keypoints(self, keypoints, transformations, **kwargs):
        return keypoints

    def _equalize_single_image(self, image):
        ops = self.backend
        if self.data_format == "channels_last":
            return ops.numpy.stack(
                [
                    self._scale_channel(image[..., c])
                    for c in range(image.shape[-1])
                ],
                axis=-1,
            )
        else:
            return ops.numpy.stack(
                [self._scale_channel(image[c]) for c in range(image.shape[-3])],
                axis=-3,
            )

    def _scale_channel(self, image_channel):
        ops = self.backend
        hist = ops.numpy.bincount(
            ops.numpy.reshape(image_channel, [-1]), minlength=self.bins
        )
        nonzero = ops.numpy.where(ops.numpy.not_equal(hist, 0), None, None)
        nonzero_hist = ops.numpy.reshape(ops.numpy.take(hist, nonzero), [-1])
        step = ops.numpy.floor_divide(
            ops.numpy.sum(hist) - nonzero_hist[-1], 255
        )

        def step_is_0():
            return ops.cast(image_channel, "uint8")

        def step_not_0():
            lut = ops.numpy.floor_divide(
                ops.numpy.add(
                    ops.numpy.cumsum(hist), ops.numpy.floor_divide(step, 2)
                ),
                step,
            )
            lut = ops.numpy.pad(lut[:-1], [[1, 0]])
            lut = ops.numpy.clip(lut, 0, 255)
            result = ops.numpy.take(lut, ops.cast(image_channel, "int64"))
            return ops.cast(result, "uint8")

        return ops.cond(step == 0, step_is_0, step_not_0)

    def get_config(self):
        config = super().get_config()
        config.update({"p": self.p, "bins": self.bins})
        return config



import numpy as np
from absl.testing import parameterized
from keras.src import testing
from keras.src.testing.test_utils import named_product

from keras_aug._src.backend.bounding_box import BoundingBoxBackend


class BoundingBoxBackendTest(testing.TestCase, parameterized.TestCase):
    size = 1000.0
    xyxy_box = np.array([[[10, 20, 110, 120], [20, 30, 120, 130]]], "float32")
    yxyx_box = np.array([[[20, 10, 120, 110], [30, 20, 130, 120]]], "float32")
    xywh_box = np.array([[[10, 20, 100, 100], [20, 30, 100, 100]]], "float32")
    center_xywh_box = np.array(
        [[[60, 70, 100, 100], [70, 80, 100, 100]]], "float32"
    )

    def get_box(self, name):
        box_dict = {
            "xyxy": self.xyxy_box,
            "yxyx": self.yxyx_box,
            "xywh": self.xywh_box,
            "center_xywh": self.center_xywh_box,
            "rel_xyxy": self.xyxy_box / self.size,
            "rel_yxyx": self.yxyx_box / self.size,
            "rel_xywh": self.xywh_box / self.size,
            "rel_center_xywh": self.center_xywh_box / self.size,
        }
        return box_dict[name]

    @parameterized.named_parameters(
        named_product(
            source=[
                "xyxy",
                "yxyx",
                "xywh",
                "center_xywh",
                "rel_xyxy",
                "rel_yxyx",
                "rel_xywh",
                "rel_center_xywh",
            ],
            target=[
                "xyxy",
                "yxyx",
                "xywh",
                "center_xywh",
                "rel_xyxy",
                "rel_yxyx",
                "rel_xywh",
                "rel_center_xywh",
            ],
        )
    )
    def test_convert_format(self, source, target):
        bbox_backend = BoundingBoxBackend()
        boxes = self.get_box(source)
        ref_boxes = self.get_box(target)

        # Test batched
        result = bbox_backend.convert_format(boxes, source, target, 1000, 1000)
        self.assertAllClose(result, ref_boxes)

        # Test unbatched
        boxes = boxes[0]
        ref_boxes = ref_boxes[0]
        result = bbox_backend.convert_format(boxes, source, target, 1000, 1000)
        self.assertAllClose(result, ref_boxes)



import keras
import numpy as np
from absl.testing import parameterized
from keras import backend
from keras.src import testing
from keras.src.testing.test_utils import named_product

from keras_aug._src.layers.vision.mix_up import MixUp
from keras_aug._src.utils.test_utils import get_images


class FixedMixUp(MixUp):
    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        lam = ops.numpy.ones([batch_size]) * 0.5
        return lam


class MixUpTest(testing.TestCase, parameterized.TestCase):
    def setUp(self):
        # Defaults to channels_last
        self.data_format = backend.image_data_format()
        backend.set_image_data_format("channels_last")
        return super().setUp()

    def tearDown(self) -> None:
        backend.set_image_data_format(self.data_format)
        return super().tearDown()

    @parameterized.named_parameters(named_product(dtype=["float32", "uint8"]))
    def test_correctness(self, dtype):
        if "float" in dtype:
            atol = 1e-6
        else:
            atol = 0.5

        # Test channels_last
        images = get_images(dtype, "channels_last")
        labels = np.array([0, 1], "float32")
        inputs = {"images": images, "labels": labels}
        layer = FixedMixUp(num_classes=2, dtype=dtype)
        outputs = layer(inputs)

        self.assertDType(outputs["images"], dtype)
        self.assertAllClose(
            outputs["images"][0], images[0] / 2.0 + images[1] / 2.0, atol=atol
        )
        self.assertAllClose(
            outputs["images"][1], images[0] / 2.0 + images[1] / 2.0, atol=atol
        )
        self.assertAllClose(outputs["labels"], [[0.5, 0.5], [0.5, 0.5]])

        # Test channels_first
        backend.set_image_data_format("channels_first")
        images = get_images(dtype, "channels_first")
        labels = np.array([0, 1], "float32")
        inputs = {"images": images, "labels": labels}
        layer = FixedMixUp(num_classes=2, dtype=dtype)
        outputs = layer(inputs)

        self.assertDType(outputs["images"], dtype)
        self.assertAllClose(
            outputs["images"][0], images[0] / 2.0 + images[1] / 2.0, atol=atol
        )
        self.assertAllClose(
            outputs["images"][1], images[0] / 2.0 + images[1] / 2.0, atol=atol
        )
        self.assertAllClose(outputs["labels"], [[0.5, 0.5], [0.5, 0.5]])

    def test_shape(self):
        # Test dynamic shape
        x = keras.KerasTensor((None, None, None, 3))
        y = MixUp()(x)
        self.assertEqual(y.shape, (None, None, None, 3))

        # Test static shape
        x = keras.KerasTensor((None, 32, 32, 3))
        y = MixUp()(x)
        self.assertEqual(y.shape, (None, 32, 32, 3))

    def test_model(self):
        layer = MixUp()
        inputs = keras.layers.Input(shape=(None, None, 3))
        outputs = layer(inputs)
        model = keras.models.Model(inputs, outputs)
        self.assertEqual(model.output_shape, (None, None, None, 3))

    def test_config(self):
        x = get_images("float32", "channels_last")
        layer = FixedMixUp()
        y = layer(x)

        layer = FixedMixUp.from_config(layer.get_config())
        y2 = layer(x)
        self.assertAllClose(y, y2)

    def test_tf_data_compatibility(self):
        import tensorflow as tf

        layer = MixUp()
        x = get_images("float32", "channels_last")
        ds = tf.data.Dataset.from_tensor_slices(x).batch(2).map(layer)
        for output in ds.take(1):
            self.assertIsInstance(output, tf.Tensor)
            self.assertEqual(output.shape, (2, 32, 32, 3))



import numbers
from collections.abc import Sequence

from keras import backend


def standardize_parameter(
    parameter,
    name="parameter",
    center=0.0,
    bound=None,
    allow_none=True,
    allow_single_number=True,
):
    if parameter is None and not allow_none:
        raise ValueError(f"`{name}` cannot be `None`")
    if parameter is None and allow_none:
        return parameter

    if not isinstance(parameter, Sequence) and not allow_single_number:
        raise ValueError(
            f"`{name}` cannot be a single number."
            f"Received: {name}={parameter}"
        )
    if not isinstance(parameter, Sequence):
        parameter = abs(parameter)
        parameter = (center - parameter, center + parameter)
    elif len(parameter) > 2:
        raise ValueError(
            f"`{name}` must be a sequence of 2 values. "
            f"Received: {name}={parameter}"
        )
    if parameter[0] > parameter[1]:
        raise ValueError(
            f"`{name}` must be in the order that first element is bigger "
            f"that second element. Received: {name}={parameter}"
        )
    if bound is not None:
        if parameter[0] < bound[0] or parameter[1] > bound[1]:
            raise ValueError(
                f"{name} is out of bounds `[{bound[0]}, {bound[1]}]`. "
                f"Received: {name}={parameter}"
            )
    return tuple(parameter)


def standardize_value_range(value_range):
    if not isinstance(value_range, Sequence) or len(value_range) != 2:
        raise ValueError(
            "`value_range` must be a sequence of numbers. "
            f"Received: value_range={value_range}"
        )
    if value_range[0] > value_range[1]:
        raise ValueError(
            "`value_range` must be in the order that first element is bigger "
            f"that second element. Received: value_range={value_range}"
        )
    return tuple(value_range)


def standardize_size(size):
    if isinstance(size, numbers.Number):
        return int(size), int(size)
    if isinstance(size, Sequence) and len(size) == 1:
        return int(size[0]), int(size[0])
    if len(size) != 2:
        raise ValueError(
            "`size` must be a single integer or the sequence of 2 "
            f"numbers. Received: size={size}"
        )
    return int(size[0]), int(size[1])


def standardize_interpolation(interpolation):
    if isinstance(interpolation, str):
        interpolation = interpolation.lower()
        if interpolation not in ("nearest", "bilinear", "bicubic"):
            raise ValueError(
                "Invalid `interpolation`. Available values are 'nearest', "
                "'bilinear' and 'bicubic'. "
                f"Received: interpolation={interpolation}"
            )
        return interpolation
    else:
        raise ValueError(
            "`interpolation` must be `str`. "
            f"Received: interpolation={interpolation} of type "
            f"{type(interpolation)}"
        )


def standardize_padding_mode(padding_mode):
    available_padding_mode = ("constant", "reflect", "symmetric")
    if padding_mode not in available_padding_mode:
        raise ValueError(
            "Invalid `padding_mode`. Available values are: "
            f"{list(available_padding_mode)}. "
            f"Received: padding_mode={padding_mode}"
        )
    return padding_mode


def standardize_bbox_format(bounding_box_format):
    if bounding_box_format is None:
        return bounding_box_format
    available_bounding_box_format = (
        "xyxy",
        "xywh",
        "center_xywh",
        "rel_xyxy",
        "rel_xywh",
        "rel_center_xywh",
    )
    if bounding_box_format not in available_bounding_box_format:
        raise ValueError(
            "Invalid `bounding_box_format`. Available values are: "
            f"{list(available_bounding_box_format)}. "
            f"Received: bounding_box_format={bounding_box_format}"
        )
    return bounding_box_format


def standardize_data_format(data_format):
    if data_format is None:
        data_format = backend.image_data_format()
    if data_format not in ("channels_last", "channels_first"):
        raise ValueError(
            "Invalid `data_format`. Available values are: "
            f"['channels_last', 'channels_first']. "
            f"Received: data_format={data_format}"
        )
    return data_format



import typing

import keras

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_parameter


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class ColorJitter(VisionRandomLayer):
    """Randomly change the brightness, contrast, saturation and hue.

    The input images must be 3 channels.

    Args:
        brightness: How much to jitter brightness. The factor will be chosen
            uniformly from `[1 - brightness, 1 + brightness]` or
            `[min, max]` if given the range of brightness. Set to `None` to
            deactivate brightness jittering. Defaults to `None`.
        contrast: How much to jitter contrast. The factor will be chosen
            uniformly from `[1 - contrast, 1 + contrast]` or
            `[min, max]` if given the range of contrast. Set to `None` to
            deactivate contrast jittering. Defaults to `None`.
        saturation: How much to jitter saturation. The factor will be chosen
            uniformly from `[1 - saturation, 1 + saturation]` or
            `[min, max]` if given the range of saturation. Set to `None` to
            deactivate saturation jittering. Defaults to `None`.
        hue: How much to jitter hue. The factor will be chosen
            uniformly from `[-hue, hue]` or `[min, max]` if given the range of
            hue. Should have `0 <= hue <= 0.5` or `-0.5 <= min <= max <= 0.5`.
            Set to `None` to deactivate hue jittering. Defaults to `None`.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(
        self,
        brightness: typing.Union[None, float, typing.Sequence[float]] = None,
        contrast: typing.Union[None, float, typing.Sequence[float]] = None,
        saturation: typing.Union[None, float, typing.Sequence[float]] = None,
        hue: typing.Union[None, float, typing.Sequence[float]] = None,
        data_format=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.brightness = standardize_parameter(
            brightness, "brightness", center=1.0, bound=(0, float("inf"))
        )
        self.contrast = standardize_parameter(
            contrast, "contrast", center=1.0, bound=(0, float("inf"))
        )
        self.saturation = standardize_parameter(
            saturation, "saturation", center=1.0, bound=(0, float("inf"))
        )
        self.hue = standardize_parameter(
            hue, "hue", center=0.0, bound=(-0.5, 0.5)
        )
        self.data_format = data_format or keras.config.image_data_format()

        if self.brightness == (0.0, 0.0):
            self.brightness = None
        if self.contrast == (0.0, 0.0):
            self.contrast = None
        if self.saturation == (0.0, 0.0):
            self.saturation = None
        if self.hue == (0.0, 0.0):
            self.hue = None

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        random_generator = self.random_generator

        fn_idx = ops.random.shuffle(
            ops.numpy.arange(4, dtype="int32"), seed=random_generator
        )
        b, c, s, h = None, None, None, None

        def generate_params(low, high):
            return ops.random.uniform(
                [batch_size], low, high, seed=random_generator
            )

        if self.brightness is not None:
            b = generate_params(self.brightness[0], self.brightness[1])
        if self.contrast is not None:
            c = generate_params(self.contrast[0], self.contrast[1])
        if self.saturation is not None:
            s = generate_params(self.saturation[0], self.saturation[1])
        if self.hue is not None:
            h = generate_params(self.hue[0], self.hue[1])

        return dict(
            fn_idx=fn_idx,
            brightness_factor=b,
            contrast_factor=c,
            saturation_factor=s,
            hue_factor=h,
        )

    def compute_output_shape(self, input_shape):
        return input_shape

    def augment_images(self, images, transformations=None, **kwargs):
        ops = self.backend

        fn_idx = transformations["fn_idx"]
        brightness_factor = transformations["brightness_factor"]
        contrast_factor = transformations["contrast_factor"]
        saturation_factor = transformations["saturation_factor"]
        hue_factor = transformations["hue_factor"]

        def adjust_brightness(images):
            if self.brightness is None:
                return images
            return self.image_backend.adjust_brightness(
                images, brightness_factor
            )

        def adjust_contrast(images):
            if self.contrast is None:
                return images
            return self.image_backend.adjust_contrast(
                images,
                contrast_factor,
                data_format=self.data_format,
            )

        def adjust_saturation(images):
            if self.saturation is None:
                return images
            return self.image_backend.adjust_saturation(
                images,
                saturation_factor,
                data_format=self.data_format,
            )

        def adjust_hue(images):
            if self.hue is None:
                return images
            images = self.image_backend.adjust_hue(
                images, hue_factor, data_format=self.data_format
            )
            return images

        branches = [
            adjust_brightness,
            adjust_contrast,
            adjust_saturation,
            adjust_hue,
        ]
        for i in range(4):
            idx = fn_idx[i]
            images = ops.core.switch(idx, branches, images)
        return images

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(self, bounding_boxes, transformations, **kwargs):
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        return segmentation_masks

    def augment_keypoints(self, keypoints, transformations, **kwargs):
        return keypoints

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "brightness": self.brightness,
                "contrast": self.contrast,
                "saturation": self.saturation,
                "hue": self.hue,
            }
        )
        return config



import keras
import numpy as np
from absl.testing import parameterized
from keras import backend
from keras.src import testing
from keras.src.testing.test_utils import named_product

from keras_aug._src.layers.vision.pad import Pad
from keras_aug._src.utils.test_utils import get_images


class PadTest(testing.TestCase, parameterized.TestCase):
    def setUp(self):
        # Defaults to channels_last
        self.data_format = backend.image_data_format()
        backend.set_image_data_format("channels_last")
        return super().setUp()

    def tearDown(self) -> None:
        backend.set_image_data_format(self.data_format)
        return super().tearDown()

    @parameterized.named_parameters(
        named_product(
            size=[(36, 36), (40, 50)],
            padding_position=[
                "border",
                "top_left",
                "top_right",
                "bottom_left",
                "bottom_right",
            ],
            padding_value=[0, 10],
            dtype=["float32", "uint8"],
        )
    )
    def test_correctness(self, size, padding_position, padding_value, dtype):
        np.random.seed(42)
        x = get_images(dtype, "channels_last")
        if dtype == "float32":
            x = np.clip(x, 0.5, 1.0)
        elif dtype == "uint8":
            x = np.clip(x, 127, 255)
        layer = Pad(
            size, "constant", padding_position, padding_value, dtype=dtype
        )
        y = layer(x)

        self.assertDType(y, dtype)
        self.assertEqual(tuple(y.shape), (2, *size, 3))
        if padding_position == "border":
            self.assertAllClose(y[0, 0, 0, 0], padding_value)
            self.assertAllClose(y[0, 0, -1, 0], padding_value)
            self.assertAllClose(y[0, -1, 0, 0], padding_value)
            self.assertAllClose(y[0, -1, -1, 0], padding_value)
        elif padding_position == "top_left":
            self.assertAllClose(y[0, 0, 0, 0], padding_value)
            self.assertAllClose(y[0, 0, -1, 0], padding_value)
            self.assertAllClose(y[0, -1, 0, 0], padding_value)
            self.assertNotAllClose(y[0, -1, -1, 0], padding_value)
        elif padding_position == "top_right":
            self.assertAllClose(y[0, 0, 0, 0], padding_value)
            self.assertAllClose(y[0, 0, -1, 0], padding_value)
            self.assertNotAllClose(y[0, -1, 0, 0], padding_value)
            self.assertAllClose(y[0, -1, -1, 0], padding_value)
        elif padding_position == "bottom_left":
            self.assertAllClose(y[0, 0, 0, 0], padding_value)
            self.assertNotAllClose(y[0, 0, -1, 0], padding_value)
            self.assertAllClose(y[0, -1, 0, 0], padding_value)
            self.assertAllClose(y[0, -1, -1, 0], padding_value)
        elif padding_position == "bottom_right":
            self.assertNotAllClose(y[0, 0, 0, 0], padding_value)
            self.assertAllClose(y[0, 0, -1, 0], padding_value)
            self.assertAllClose(y[0, -1, 0, 0], padding_value)
            self.assertAllClose(y[0, -1, -1, 0], padding_value)

    @parameterized.named_parameters(
        named_product(mode=["constant", "reflect", "symmetric"])
    )
    def test_mode(self, mode):
        if backend.backend() == "torch" and mode == "symmetric":
            self.skipTest("TODO: Need to investigate")
        np.random.seed(42)
        x = get_images("float32", "channels_last")
        x = np.clip(x, 0.5, 1.0)
        layer = Pad(48, padding_mode=mode)
        y = layer(x)

        pad_width = [[0, 0], [8, 8], [8, 8], [0, 0]]
        ref_y = np.pad(x, pad_width, mode=mode)
        self.assertAllClose(y, ref_y)

    def test_shape(self):
        # Test dynamic shape
        x = keras.KerasTensor((None, None, None, 3))
        y = Pad(16)(x)
        self.assertEqual(y.shape, (None, None, None, 3))

        # Test static shape
        x = keras.KerasTensor((None, 32, 32, 3))
        y = Pad(48)(x)
        self.assertEqual(y.shape, (None, 48, 48, 3))

    def test_model(self):
        # Test dynamic shape
        layer = Pad(16)
        inputs = keras.layers.Input(shape=[None, None, 3])
        outputs = layer(inputs)
        model = keras.models.Model(inputs, outputs)
        self.assertEqual(model.output_shape, (None, None, None, 3))

        # Test static shape
        layer = Pad((32, 48))
        inputs = keras.layers.Input(shape=[32, 32, 3])
        outputs = layer(inputs)
        model = keras.models.Model(inputs, outputs)
        self.assertEqual(model.output_shape, (None, 32, 48, 3))

    def test_config(self):
        x = get_images("float32", "channels_last")
        layer = Pad((32, 48))
        y = layer(x)

        layer = Pad.from_config(layer.get_config())
        y2 = layer(x)
        self.assertAllClose(y, y2)

    def test_tf_data_compatibility(self):
        import tensorflow as tf

        layer = Pad(48)
        x = get_images("float32", "channels_last")
        ds = tf.data.Dataset.from_tensor_slices(x).batch(2).map(layer)
        for output in ds.take(1):
            self.assertIsInstance(output, tf.Tensor)
            self.assertEqual(output.shape, (2, 48, 48, 3))

    def test_augment_bounding_box(self):
        # Test full bounding boxes
        images = np.zeros([1, 20, 20, 3]).astype("float32")
        boxes = {
            "boxes": np.array(
                [
                    [[0, 0, 1, 1], [0, 0, 1, 1]],
                    [[0, 0, 1, 1], [0, 0, 1, 1]],
                ],
                "float32",
            ),
            "classes": np.array([[0, 0], [0, 0]], "float32"),
        }
        input = {"images": images, "bounding_boxes": boxes}
        layer = Pad(30, bounding_box_format="rel_xyxy")

        output = layer(input)
        self.assertNotAllClose(
            output["bounding_boxes"]["boxes"], boxes["boxes"]
        )
        self.assertAllClose(
            output["bounding_boxes"]["classes"], boxes["classes"]
        )

        # Test arbitrary bounding boxes
        images = np.zeros([1, 20, 20, 3]).astype("float32")
        boxes = {
            "boxes": np.array(
                [
                    [[3, 4, 17, 18], [10, 12, 16, 19]],
                    [[0, 0, 1, 1], [15, 12, 17, 20]],
                ],
                "float32",
            ),
            "classes": np.array([[0, 1], [2, 3]], "float32"),
        }
        input = {"images": images, "bounding_boxes": boxes}
        layer = Pad(30, bounding_box_format="xyxy")

        output = layer(input)
        expected_boxes = {
            "boxes": np.array(
                [
                    [[8, 9, 22, 23], [15, 17, 21, 24]],
                    [[5, 5, 6, 6], [20, 17, 22, 25]],
                ],
                "float32",
            ),
            "classes": np.array([[0, 1], [2, 3]], "float32"),
        }
        self.assertAllClose(
            output["bounding_boxes"]["boxes"], expected_boxes["boxes"]
        )
        self.assertAllClose(
            output["bounding_boxes"]["classes"], expected_boxes["classes"]
        )

    def test_augment_segmentation_mask(self):
        num_classes = 8
        images_shape = (1, 32, 32, 3)
        masks_shape = (1, 32, 32, 1)
        images = np.random.uniform(size=images_shape).astype("float32")
        masks = np.random.randint(2, size=masks_shape) * (num_classes - 1) + 1
        inputs = {"images": images, "segmentation_masks": masks}

        layer = Pad(48)
        output = layer(inputs)
        output_masks = output["segmentation_masks"][:, 8:-8, 8:-8, :]
        output_masks_border = output["segmentation_masks"][:, :8, :8, :]
        self.assertAllClose(output_masks, masks)
        self.assertAllClose(
            output_masks_border, np.ones_like(output_masks_border) * -1
        )



import typing

import keras

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer
from keras_aug._src.utils.argument_validation import standardize_bbox_format
from keras_aug._src.utils.argument_validation import standardize_data_format
from keras_aug._src.utils.argument_validation import standardize_padding_mode
from keras_aug._src.utils.argument_validation import standardize_size


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class Pad(VisionRandomLayer):
    """Pad the inputs to the given size.

    Args:
        size: Desired output size. If `size` is a sequence like `(h, w)`,
            output size will be matched to this. If `size` is an int, a square
            size (size, size) is made.
        padding_mode: The mode of the padding. Available values:
            `"constant", "edge", "reflect", "symmetric"`. Defaults to
            `"constant"`.
        padding_position: The position of the padding. Available values:
            `"border", "top_left", "top_right", "bottom_left", "bottom_right".`
            Defaults to `"border"`.
        padding_value: The padding value. It only affects when
            `padding_mode="constant"`. Defaults to `0`.
        bounding_box_format: The format of the bounding boxes. If specified,
            the available values are `"xyxy", "xywh", "center_xywh", "rel_xyxy",
            "rel_xywh", "rel_center_xywh"`. Defaults to `None`.
        data_format: A string specifying the data format of the input images.
            It can be either `"channels_last"` or `"channels_first"`.
            If not specified, the value will be interpreted by
            `keras.config.image_data_format`. Defaults to `None`.
    """

    def __init__(
        self,
        size: typing.Union[typing.Sequence[int], int],
        padding_mode: str = "constant",
        padding_position: str = "border",
        padding_value: float = 0,
        bounding_box_format=None,
        data_format: typing.Optional[str] = None,
        **kwargs,
    ):
        super().__init__(has_generator=False, **kwargs)
        available_padding_position = (
            "border",
            "top_left",
            "top_right",
            "bottom_left",
            "bottom_right",
        )
        if padding_position not in available_padding_position:
            raise ValueError(
                "Invalid `padding_position`. Available values are: "
                f"{list(available_padding_position)}. "
                f"Received: padding_position={padding_position}"
            )

        self.size = standardize_size(size)
        self.padding_mode = standardize_padding_mode(padding_mode)
        self.padding_position = padding_position
        self.padding_value = padding_value
        self.bounding_box_format = standardize_bbox_format(bounding_box_format)
        self.data_format = standardize_data_format(data_format)

        if self.data_format == "channels_last":
            self.h_axis, self.w_axis = -3, -2
        else:
            self.h_axis, self.w_axis = -2, -1

    def compute_output_shape(self, input_shape):
        # Get original h, w
        if not isinstance(input_shape, dict):
            h, w = input_shape[self.h_axis], input_shape[self.w_axis]
        else:
            images_shape = list(input_shape[self.IMAGES])
            h, w = images_shape[self.h_axis], images_shape[self.w_axis]

        # Compute new_h and new_w
        new_h, new_w = None, None
        if h is not None:
            new_h = self.size[0] if self.size[0] > h else h
        if w is not None:
            new_w = self.size[1] if self.size[1] > w else w

        # Update
        if not isinstance(input_shape, dict):
            output_shape = list(input_shape)
            output_shape[self.h_axis] = new_h
            output_shape[self.w_axis] = new_w
        else:
            output_shape = input_shape.copy()
            output_shape[self.IMAGES][self.h_axis] = new_h
            output_shape[self.IMAGES][self.w_axis] = new_w
        return output_shape

    def get_params(self, batch_size, images=None, **kwargs):
        ops = self.backend
        images_shape = ops.shape(images)
        height, width = images_shape[self.h_axis], images_shape[self.w_axis]

        pad_h = ops.numpy.where(self.size[0] > height, self.size[0] - height, 0)
        pad_w = ops.numpy.where(self.size[1] > width, self.size[1] - width, 0)
        pad_h = ops.cast(pad_h, "int32")
        pad_w = ops.cast(pad_w, "int32")

        if self.padding_position == "border":
            pad_top = ops.cast(ops.cast(pad_h, "float32") / 2, "int32")
            pad_bottom = pad_h - pad_top
            pad_left = ops.cast(ops.cast(pad_w, "float32") / 2, "int32")
            pad_right = pad_w - pad_left
        elif self.padding_position == "top_left":
            pad_top = pad_h
            pad_bottom = 0
            pad_left = pad_w
            pad_right = 0
        elif self.padding_position == "top_right":
            pad_top = pad_h
            pad_bottom = 0
            pad_left = 0
            pad_right = pad_w
        elif self.padding_position == "bottom_left":
            pad_top = 0
            pad_bottom = pad_h
            pad_left = pad_w
            pad_right = 0
        elif self.padding_position == "bottom_right":
            pad_top = 0
            pad_bottom = pad_h
            pad_left = 0
            pad_right = pad_w
        else:
            raise NotImplementedError

        return {
            "pad_top": pad_top,
            "pad_bottom": pad_bottom,
            "pad_left": pad_left,
            "pad_right": pad_right,
        }

    def augment_images(self, images, transformations, **kwargs):
        ops = self.backend

        # Pad
        pad_top = transformations["pad_top"]
        pad_bottom = transformations["pad_bottom"]
        pad_left = transformations["pad_left"]
        pad_right = transformations["pad_right"]
        images = self.image_backend.pad(
            images,
            self.padding_mode,
            pad_top,
            pad_bottom,
            pad_left,
            pad_right,
            self.padding_value,
            self.data_format,
        )
        return ops.cast(images, self.compute_dtype)

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(
        self,
        bounding_boxes,
        transformations,
        images=None,
        raw_images=None,
        **kwargs,
    ):
        if self.bounding_box_format is None:
            raise ValueError(
                f"{self.__class__.__name__} was called with bounding boxes,"
                "but no `bounding_box_format` was specified in the constructor."
            )
        ops = self.backend

        ori_images_shape = ops.shape(raw_images)
        ori_height = ori_images_shape[self.h_axis]
        ori_width = ori_images_shape[self.w_axis]

        images_shape = ops.shape(images)
        height = images_shape[self.h_axis]
        width = images_shape[self.w_axis]

        bounding_boxes = self.bbox_backend.convert_format(
            bounding_boxes,
            source=self.bounding_box_format,
            target="xyxy",
            height=ori_height,
            width=ori_width,
        )

        pad_top = ops.cast(transformations["pad_top"], dtype="float32")
        pad_left = ops.cast(transformations["pad_left"], dtype="float32")
        boxes = self.bbox_backend.pad(
            bounding_boxes["boxes"], pad_top, pad_left
        )

        bounding_boxes = bounding_boxes.copy()
        bounding_boxes["boxes"] = boxes
        bounding_boxes = self.bbox_backend.convert_format(
            bounding_boxes,
            source=self.bounding_box_format,
            target="xyxy",
            height=height,
            width=width,
        )
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        ops = self.backend
        images_shape = ops.shape(segmentation_masks)

        # Pad
        pad_top = transformations["pad_top"]
        pad_bottom = transformations["pad_bottom"]
        pad_left = transformations["pad_left"]
        pad_right = transformations["pad_right"]
        if self._backend.name == "torch":  # Workaround for torch
            pad_top = int(pad_top)
            pad_bottom = int(pad_bottom)
            pad_left = int(pad_left)
            pad_right = int(pad_right)
        pad_width = [[pad_top, pad_bottom], [pad_left, pad_right]]
        if self.data_format == "channels_last":
            pad_width = pad_width + [[0, 0]]
        else:
            pad_width = [[0, 0]] + pad_width
        if len(images_shape) == 4:
            pad_width = [[0, 0]] + pad_width
        segmentation_masks = ops.numpy.pad(
            segmentation_masks, pad_width, "constant", -1
        )
        return segmentation_masks

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "size": self.size,
                "padding_mode": self.padding_mode,
                "padding_position": self.padding_position,
                "padding_value": self.padding_value,
                "bounding_box_format": self.bounding_box_format,
            }
        )
        return config



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_aug._src.layers.vision.center_crop import CenterCrop
from keras_aug._src.layers.vision.color_jitter import ColorJitter
from keras_aug._src.layers.vision.cut_mix import CutMix
from keras_aug._src.layers.vision.gaussian_blur import GaussianBlur
from keras_aug._src.layers.vision.gaussian_noise import GaussianNoise
from keras_aug._src.layers.vision.identity import Identity
from keras_aug._src.layers.vision.max_bounding_box import MaxBoundingBox
from keras_aug._src.layers.vision.mix_up import MixUp
from keras_aug._src.layers.vision.mosaic import Mosaic
from keras_aug._src.layers.vision.normalize import Normalize
from keras_aug._src.layers.vision.pad import Pad
from keras_aug._src.layers.vision.rand_augment import RandAugment
from keras_aug._src.layers.vision.random_affine import RandomAffine
from keras_aug._src.layers.vision.random_auto_contrast import RandomAutoContrast
from keras_aug._src.layers.vision.random_channel_permutation import (
    RandomChannelPermutation,
)
from keras_aug._src.layers.vision.random_crop import RandomCrop
from keras_aug._src.layers.vision.random_equalize import RandomEqualize
from keras_aug._src.layers.vision.random_erasing import RandomErasing
from keras_aug._src.layers.vision.random_flip import RandomFlip
from keras_aug._src.layers.vision.random_grayscale import RandomGrayscale
from keras_aug._src.layers.vision.random_hsv import RandomHSV
from keras_aug._src.layers.vision.random_invert import RandomInvert
from keras_aug._src.layers.vision.random_posterize import RandomPosterize
from keras_aug._src.layers.vision.random_resized_crop import RandomResizedCrop
from keras_aug._src.layers.vision.random_rotation import RandomRotation
from keras_aug._src.layers.vision.random_sharpen import RandomSharpen
from keras_aug._src.layers.vision.random_solarize import RandomSolarize
from keras_aug._src.layers.vision.rescale import Rescale
from keras_aug._src.layers.vision.resize import Resize
from keras_aug._src.layers.vision.trivial_augment import TrivialAugmentWide



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_aug._src.ops.image import adjust_brightness
from keras_aug._src.ops.image import adjust_contrast
from keras_aug._src.ops.image import adjust_hue
from keras_aug._src.ops.image import adjust_saturation
from keras_aug._src.ops.image import affine
from keras_aug._src.ops.image import auto_contrast
from keras_aug._src.ops.image import blend
from keras_aug._src.ops.image import crop
from keras_aug._src.ops.image import equalize
from keras_aug._src.ops.image import guassian_blur
from keras_aug._src.ops.image import invert
from keras_aug._src.ops.image import pad
from keras_aug._src.ops.image import posterize
from keras_aug._src.ops.image import rgb_to_grayscale
from keras_aug._src.ops.image import sharpen
from keras_aug._src.ops.image import solarize
from keras_aug._src.ops.image import transform_dtype



import numpy as np
from keras import backend
from keras import ops

from keras_aug._src import ops as ka_ops
from keras_aug._src.keras_aug_export import keras_aug_export


@keras_aug_export(parent_path=["keras_aug.visualization"])
def draw_bounding_boxes(
    images,
    bounding_boxes,
    bounding_box_format,
    class_mapping=None,
    color_mapping=None,
    thickness=1,
    font_scale=1.0,
    data_format=None,
):
    try:
        import cv2
    except ImportError:
        raise ImportError(
            "Cannot import OpenCV. You can install it by "
            "`pip install opencv-python`."
        )
    class_mapping = class_mapping or {}
    if len(class_mapping) > 0:
        num_classes = len(class_mapping)
    else:
        num_classes = 80  # Defaults to 80 (COCO)
    if color_mapping is None:
        color_mapping = {}
        for i, color in enumerate(_generate_color_palette(num_classes)):
            color_mapping[i] = color
    thickness = int(thickness)
    data_format = data_format or backend.image_data_format()
    images_shape = ops.shape(images)
    if len(images_shape) != 4:
        raise ValueError(
            "`images` must be batched 4D tensor. "
            f"Received: images.shape={images_shape}"
        )
    if not isinstance(bounding_boxes, dict):
        raise TypeError(
            "`bounding_boxes` should be a dict. "
            f"Received: bounding_boxes={bounding_boxes} of type "
            f"{type(bounding_boxes)}"
        )
    if "boxes" not in bounding_boxes or "classes" not in bounding_boxes:
        raise ValueError(
            "`bounding_boxes` should be a dict containing 'boxes' and "
            f"'classes' keys. Received: bounding_boxes={bounding_boxes}"
        )
    if data_format == "channels_last":
        h_axis = -3
        w_axis = -2
    else:
        h_axis = -2
        w_axis = -1
    height = images_shape[h_axis]
    width = images_shape[w_axis]
    bounding_boxes = bounding_boxes.copy()
    bounding_boxes = ka_ops.bounding_box.convert_format(
        bounding_boxes, bounding_box_format, "xyxy", height, width
    )

    # To numpy array
    images = ka_ops.image.transform_dtype(images, images.dtype, "uint8")
    images = ops.convert_to_numpy(images)
    boxes = ops.convert_to_numpy(bounding_boxes["boxes"])
    classes = ops.convert_to_numpy(bounding_boxes["classes"])
    if "confidences" in bounding_boxes:
        confidences = ops.convert_to_numpy(bounding_boxes["confidences"])
    else:
        confidences = None

    result = []
    batch_size = images.shape[0]
    for i in range(batch_size):
        _image = images[i]
        _box = boxes[i]
        _class = classes[i]
        for box_i in range(_box.shape[0]):
            x1, y1, x2, y2 = _box[box_i].astype("int32")
            c = _class[box_i].astype("int32")
            if c == -1:
                continue
            x1, y1, x2, y2 = int(x1), int(y1), int(x2), int(y2)
            c = int(c)
            color = color_mapping[c % num_classes]

            # Draw bounding box
            cv2.rectangle(_image, (x1, y1), (x2, y2), color, thickness)

            if c in class_mapping:
                label = class_mapping[c]
                if confidences is not None:
                    conf = confidences[i][box_i]
                    label = f"{label} | {conf:.2f}"

                font_x1, font_y1 = _find_text_location(
                    x1, y1, font_scale, thickness
                )
                cv2.putText(
                    _image,
                    label,
                    (font_x1, font_y1),
                    cv2.FONT_HERSHEY_SIMPLEX,
                    font_scale,
                    color,
                    thickness,
                )
        result.append(_image)
    return np.stack(result, axis=0)


def _find_text_location(x, y, font_scale, thickness):
    font_height = int(font_scale * 12)
    target_y = y - 8
    if target_y - (2 * font_height) > 0:
        return x, y - 8

    line_offset = thickness
    static_offset = 3

    return (
        x + static_offset,
        y + (2 * font_height) + line_offset + static_offset,
    )


def _generate_color_palette(num_classes: int):
    palette = np.array([2**25 - 1, 2**15 - 1, 2**21 - 1])
    return [((i * palette) % 255).tolist() for i in range(num_classes)]



import keras
from keras import backend

from keras_aug._src.keras_aug_export import keras_aug_export
from keras_aug._src.layers.base.vision_random_layer import VisionRandomLayer


@keras_aug_export(parent_path=["keras_aug.layers.vision"])
@keras.saving.register_keras_serializable(package="keras_aug")
class Rescale(VisionRandomLayer):
    """Rescales the values of the images to a new range

    The rescaling equation: `y = x * scale + offset`.

    Args:
        scale: The scale to apply to the images.
        offset: The offset to apply to the images. Defaults to `0.0`
    """

    def __init__(self, scale: float, offset: float = 0.0, **kwargs):
        super().__init__(has_generator=False, **kwargs)
        self.scale = float(scale)
        self.offset = float(offset)

        if not backend.is_float_dtype(self.compute_dtype):
            dtype = self.dtype_policy
            raise ValueError(
                f"The `dtype` of '{self.__class__.__name__}' must be float. "
                f"Received: dtype={dtype}"
            )

    def compute_output_shape(self, input_shape):
        return input_shape

    def augment_images(self, images, transformations, **kwargs):
        ops = self.backend
        original_dtype = backend.standardize_dtype(images.dtype)
        images = self.image_backend.transform_dtype(
            images, images.dtype, backend.result_type(images.dtype, float)
        )
        scale = ops.convert_to_tensor(self.scale, images.dtype)
        offset = ops.convert_to_tensor(self.offset, images.dtype)
        images = ops.numpy.add(ops.numpy.multiply(images, scale), offset)
        images = self.image_backend.transform_dtype(
            images, images.dtype, original_dtype
        )
        return images

    def augment_labels(self, labels, transformations, **kwargs):
        return labels

    def augment_bounding_boxes(self, bounding_boxes, transformations, **kwargs):
        return bounding_boxes

    def augment_segmentation_masks(
        self, segmentation_masks, transformations, **kwargs
    ):
        return segmentation_masks

    def augment_keypoints(self, keypoints, transformations, **kwargs):
        return keypoints

    def get_config(self):
        config = super().get_config()
        config.update({"scale": self.scale, "offset": self.offset})
        return config



import numpy as np
from keras import backend
from keras import ops

from keras_aug._src import ops as ka_ops
from keras_aug._src.keras_aug_export import keras_aug_export


@keras_aug_export(parent_path=["keras_aug.visualization"])
def draw_segmentation_masks(
    images,
    segmentation_masks,
    num_classes=None,
    color_mapping=None,
    alpha=0.8,
    ignore_index=-1,
    data_format=None,
):
    data_format = data_format or backend.image_data_format()
    images_shape = ops.shape(images)
    if len(images_shape) != 4:
        raise ValueError(
            "`images` must be batched 4D tensor. "
            f"Received: images.shape={images_shape}"
        )
    images = ops.convert_to_tensor(images)
    images = ka_ops.image.transform_dtype(images, images.dtype, "float32")
    segmentation_masks = ops.convert_to_tensor(segmentation_masks)

    if not backend.is_int_dtype(segmentation_masks.dtype):
        dtype = backend.standardize_dtype(segmentation_masks.dtype)
        raise TypeError(
            "`segmentation_masks` must be in integer dtype. "
            f"Received: segmentation_masks.dtype={dtype}"
        )

    # Infer num_classes
    if num_classes is None:
        num_classes = int(ops.convert_to_numpy(ops.max(segmentation_masks)))
    if color_mapping is None:
        colors = _generate_color_palette(num_classes)
    else:
        colors = [color_mapping[i] for i in range(num_classes)]
    valid_masks = ops.not_equal(segmentation_masks, ignore_index)
    valid_masks = ops.squeeze(valid_masks, axis=-1)
    segmentation_masks = ops.nn.one_hot(segmentation_masks, num_classes)
    segmentation_masks = segmentation_masks[..., 0, :]
    segmentation_masks = ops.convert_to_numpy(segmentation_masks)

    # Replace class with color
    masks = segmentation_masks
    masks = np.transpose(masks, axes=(3, 0, 1, 2)).astype("bool")
    images_to_draw = ops.convert_to_numpy(images).copy()
    for mask, color in zip(masks, colors):
        color = np.array(color, dtype=images_to_draw.dtype)
        images_to_draw[mask, ...] = color[None, :]
    images_to_draw = ops.convert_to_tensor(images_to_draw)
    images_to_draw = ka_ops.image.transform_dtype(
        images_to_draw, "uint8", "float32"
    )

    # Apply blending
    outputs = images * (1 - alpha) + images_to_draw * alpha
    outputs = ops.where(valid_masks[..., None], outputs, images)
    outputs = ka_ops.image.transform_dtype(outputs, "float32", "uint8")
    outputs = ops.convert_to_numpy(outputs)
    return outputs


def _generate_color_palette(num_classes: int):
    palette = np.array([2**25 - 1, 2**15 - 1, 2**21 - 1])
    return [((i * palette) % 255).tolist() for i in range(num_classes)]



try:
    import namex
except ImportError:
    namex = None

# These dicts reference "canonical names" only
# (i.e. the first name an object was registered with).
REGISTERED_NAMES_TO_OBJS = {}
REGISTERED_OBJS_TO_NAMES = {}


def register_internal_serializable(path, symbol):
    global REGISTERED_NAMES_TO_OBJS
    if isinstance(path, (list, tuple)):
        name = path[0]
    else:
        name = path
    REGISTERED_NAMES_TO_OBJS[name] = symbol
    REGISTERED_OBJS_TO_NAMES[symbol] = name


def get_symbol_from_name(name):
    return REGISTERED_NAMES_TO_OBJS.get(name, None)


def get_name_from_symbol(symbol):
    return REGISTERED_OBJS_TO_NAMES.get(symbol, None)


if namex:

    class keras_aug_export:
        def __init__(self, parent_path):
            package = "keras_aug"

            if isinstance(parent_path, str):
                export_paths = [parent_path]
            elif isinstance(parent_path, list):
                export_paths = parent_path
            else:
                raise ValueError(
                    f"Invalid type for `parent_path` argument: "
                    f"Received '{parent_path}' "
                    f"of type {type(parent_path)}"
                )
            for p in export_paths:
                if not p.startswith(package):
                    raise ValueError(
                        "All `export_path` values should start with "
                        f"'{package}.'. Received: parent_path={parent_path}"
                    )
            self.package = package
            self.parent_path = parent_path

        def __call__(self, symbol):
            if hasattr(symbol, "_api_export_path") and (
                symbol._api_export_symbol_id == id(symbol)
            ):
                raise ValueError(
                    f"Symbol {symbol} is already exported as "
                    f"'{symbol._api_export_path}'. "
                    f"Cannot also export it to '{self.parent_path}'."
                )
            if isinstance(self.parent_path, list):
                path = [p + f".{symbol.__name__}" for p in self.parent_path]
            elif isinstance(self.parent_path, str):
                path = self.parent_path + f".{symbol.__name__}"
            symbol._api_export_path = path
            symbol._api_export_symbol_id = id(symbol)

            register_internal_serializable(path, symbol)
            return symbol

else:

    class keras_aug_export:
        def __init__(self, parent_path):
            self.parent_path = parent_path

        def __call__(self, symbol):
            if isinstance(self.parent_path, list):
                path = [p + f".{symbol.__name__}" for p in self.parent_path]
            elif isinstance(self.parent_path, str):
                path = self.parent_path + f".{symbol.__name__}"

            register_internal_serializable(path, symbol)
            return symbol



from keras import backend
from keras import random


class DynamicBackend:
    def __init__(self, name=None):
        if name is not None and not isinstance(name, str):
            raise TypeError
        self._name = name

        # Variable
        self._backend = None

        # Init
        self.set_backend(self._name, force=True)

    @property
    def name(self):
        return self._name

    @property
    def backend(self):
        return self._backend

    def set_backend(self, name=None, force=False):
        name = name or backend.backend()
        self._backend = get_backend(name)
        self._name = name

    def reset(self):
        self.set_backend()


class DynamicRandomGenerator:
    def __init__(self, name=None, seed=None):
        if name is not None and not isinstance(name, str):
            raise TypeError
        self._name = name
        self._seed = seed

        # Variable
        self._cached_random_generator = {}

        # Init
        self.set_generator(self._name)

    @property
    def name(self):
        return self._name

    @property
    def random_generator(self):
        return self._cached_random_generator[self._name]

    def set_generator(self, name=None):
        name = name or backend.backend()
        if name in self._cached_random_generator:
            return
        self._cached_random_generator[name] = random.SeedGenerator(
            seed=self._seed, backend=get_backend(name)
        )

    def reset(self):
        self.set_generator()


def get_backend(name=None):
    name = name or backend.backend()
    if name == "tensorflow":
        import keras.src.backend.tensorflow as module
    elif name == "jax":
        import keras.src.backend.jax as module
    elif name == "torch":
        import keras.src.backend.torch as module
    elif name == "numpy":
        if backend.backend() == "numpy":
            import keras.src.backend as module
        else:
            raise NotImplementedError(
                "Currently, we cannot dynamically import the numpy backend "
                "because it would disrupt the namespace of the import."
            )
    else:
        raise NotImplementedError
    return module
